@@ -12,3 +12,11 @@ gh_path()
 {
     echo "$1" >> $GITHUB_PATH
 }
+
+gh_output()
+{
+    while [ "x$1" != "x" ]; do
+        echo "::set-output name=$1::$(eval echo \$$1)"
+        shift;
+    done
+}
