@@ -231,18 +231,20 @@ extern int filter_expr_debug;
     KW_OR = 10526,                 /* KW_OR  */
     KW_AND = 10527,                /* KW_AND  */
     KW_NOT = 10528,                /* KW_NOT  */
-    KW_LT = 10529,                 /* KW_LT  */
-    KW_LE = 10530,                 /* KW_LE  */
-    KW_EQ = 10531,                 /* KW_EQ  */
-    KW_NE = 10532,                 /* KW_NE  */
-    KW_GE = 10533,                 /* KW_GE  */
-    KW_GT = 10534,                 /* KW_GT  */
-    KW_NUM_LT = 10535,             /* KW_NUM_LT  */
-    KW_NUM_LE = 10536,             /* KW_NUM_LE  */
-    KW_NUM_EQ = 10537,             /* KW_NUM_EQ  */
-    KW_NUM_NE = 10538,             /* KW_NUM_NE  */
-    KW_NUM_GE = 10539,             /* KW_NUM_GE  */
-    KW_NUM_GT = 10540              /* KW_NUM_GT  */
+    KW_STR_LT = 10529,             /* KW_STR_LT  */
+    KW_STR_LE = 10530,             /* KW_STR_LE  */
+    KW_STR_EQ = 10531,             /* KW_STR_EQ  */
+    KW_STR_NE = 10532,             /* KW_STR_NE  */
+    KW_STR_GE = 10533,             /* KW_STR_GE  */
+    KW_STR_GT = 10534,             /* KW_STR_GT  */
+    KW_TA_LT = 10535,              /* KW_TA_LT  */
+    KW_TA_LE = 10536,              /* KW_TA_LE  */
+    KW_TA_EQ = 10537,              /* KW_TA_EQ  */
+    KW_TA_NE = 10538,              /* KW_TA_NE  */
+    KW_TA_GE = 10539,              /* KW_TA_GE  */
+    KW_TA_GT = 10540,              /* KW_TA_GT  */
+    KW_TAV_EQ = 10541,             /* KW_TAV_EQ  */
+    KW_TAV_NE = 10542              /* KW_TAV_NE  */
   };
   typedef enum filter_expr_tokentype filter_expr_token_kind_t;
 #endif
@@ -412,18 +414,20 @@ extern int filter_expr_debug;
 #define KW_OR 10526
 #define KW_AND 10527
 #define KW_NOT 10528
-#define KW_LT 10529
-#define KW_LE 10530
-#define KW_EQ 10531
-#define KW_NE 10532
-#define KW_GE 10533
-#define KW_GT 10534
-#define KW_NUM_LT 10535
-#define KW_NUM_LE 10536
-#define KW_NUM_EQ 10537
-#define KW_NUM_NE 10538
-#define KW_NUM_GE 10539
-#define KW_NUM_GT 10540
+#define KW_STR_LT 10529
+#define KW_STR_LE 10530
+#define KW_STR_EQ 10531
+#define KW_STR_NE 10532
+#define KW_STR_GE 10533
+#define KW_STR_GT 10534
+#define KW_TA_LT 10535
+#define KW_TA_LE 10536
+#define KW_TA_EQ 10537
+#define KW_TA_NE 10538
+#define KW_TA_GE 10539
+#define KW_TA_GT 10540
+#define KW_TAV_EQ 10541
+#define KW_TAV_NE 10542
 
 /* Value type.  */
 #if ! defined FILTER_EXPR_STYPE && ! defined FILTER_EXPR_STYPE_IS_DECLARED
