@@ -61,7 +61,7 @@ log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_op
   msg_trace(">>>>>> filter rule evaluation begin",
             evt_tag_str("rule", self->name),
             log_pipe_location_tag(s),
-            evt_tag_printf("msg", "%p", msg));
+            evt_tag_msg_reference(msg));
 
   res = filter_expr_eval_root(self->expr, &msg, path_options);
 
@@ -83,14 +83,16 @@ log_filter_pipe_queue(LogPipe *s, LogMessage *msg, const LogPathOptions *path_op
             evt_tag_str("result", filter_result),
             evt_tag_str("rule", self->name),
             log_pipe_location_tag(s),
-            evt_tag_printf("msg", "%p", msg));
+            evt_tag_msg_reference(msg));
 }
 
 static LogPipe *
 log_filter_pipe_clone(LogPipe *s)
 {
   LogFilterPipe *self = (LogFilterPipe *) s;
-  LogPipe *cloned = log_filter_pipe_new(filter_expr_ref(self->expr), s->cfg);
+  FilterExprNode *expr = filter_expr_clone(self->expr);
+
+  LogPipe *cloned = log_filter_pipe_new(expr, s->cfg);
   ((LogFilterPipe *)cloned)->name = g_strdup(self->name);
   return cloned;
 }
