@@ -30,14 +30,6 @@
 #include "compat/string.h"
 #include "compat/pcre.h"
 
-static gboolean
-_shall_set_values_indirectly(NVHandle value_handle)
-{
-  return value_handle != LM_V_NONE &&
-         !log_msg_is_handle_macro(value_handle) &&
-         !log_msg_is_handle_match(value_handle);
-}
-
 static void
 log_matcher_store_pattern(LogMatcher *self, const gchar *pattern)
 {
@@ -248,7 +240,7 @@ log_matcher_glob_match(LogMatcher *s, LogMessage *msg, gint value_handle, const
       if (G_UNLIKELY(!warned && (msg->flags & LF_UTF8) == 0))
         {
           msg_warning("Input is valid utf8, but the log message is not tagged as such, this performs worse than enabling validate-utf8 flag on input",
-                      evt_tag_printf("value", "%.*s", (gint) value_len, value));
+                      evt_tag_mem("value", value, value_len));
           warned = TRUE;
         }
       APPEND_ZERO(buf, value, value_len);
@@ -257,7 +249,7 @@ log_matcher_glob_match(LogMatcher *s, LogMessage *msg, gint value_handle, const
   else
     {
       msg_warning("Input is not valid utf8, glob match requires utf8 input, thus it never matches in this case",
-                  evt_tag_printf("value", "%.*s", (gint) value_len, value));
+                  evt_tag_mem("value", value, value_len));
     }
   return FALSE;
 }
@@ -391,60 +383,83 @@ log_matcher_pcre_re_compile(LogMatcher *s, const gchar *re, GError **error)
   return TRUE;
 }
 
+typedef struct _LogMatcherPcreMatchResult
+{
+  NVHandle source_handle;
+  const gchar *source_value;
+  gssize source_value_len;
+  gint *matches;
+  gint num_matches;
+} LogMatcherPcreMatchResult;
+
+static inline void
+log_matcher_pcre_re_save_source_value_to_avoid_clobbering(LogMatcherPcreMatchResult *result)
+{
+  GString *source_value_scratch = scratch_buffers_alloc();
+  g_string_assign_len(source_value_scratch, result->source_value, result->source_value_len);
+  result->source_value = source_value_scratch->str;
+
+  /* source_value_scratch will be freed automatically by the scratch-buffers GC */
+}
+
+static inline void
+log_matcher_pcre_re_feed_value(LogMatcherPcreRe *self, LogMessage *msg,
+                               NVHandle target_handle,
+                               LogMatcherPcreMatchResult *result,
+                               gint begin_index, gint end_index)
+{
+  gboolean indirect = result->source_handle != LM_V_NONE &&
+                      log_msg_is_handle_settable_with_an_indirect_value(target_handle) &&
+                      log_msg_is_handle_referencable_from_an_indirect_value(result->source_handle);
+
+  if (target_handle == result->source_handle)
+    {
+      /* we are clobbering our input at this point, e.g.  we are setting the
+       * name-value pair that we use as input.  And we use a borrowed
+       * reference so that we avoid copying.  This means that if
+       * target_handle == source_handle we would implicitly overwrite the
+       * memory area that "value" points to (to add to the injury and to
+       * make it less appearent, this is not always the case: it does not
+       * happen if the new value does not fit the old NVEntry).  */
+      log_matcher_pcre_re_save_source_value_to_avoid_clobbering(result);
+    }
+
+  if (indirect)
+    log_msg_set_value_indirect(msg, target_handle, result->source_handle, begin_index, end_index - begin_index);
+  else
+    log_msg_set_value(msg, target_handle, &result->source_value[begin_index], end_index - begin_index);
+}
+
 static void
-log_matcher_pcre_re_feed_backrefs(LogMatcher *s, LogMessage *msg, gint value_handle, int *matches, gint match_num,
-                                  const gchar *value)
+log_matcher_pcre_re_feed_backrefs(LogMatcherPcreRe *self, LogMessage *msg, LogMatcherPcreMatchResult *result)
 {
   gint i;
-  gboolean indirect = _shall_set_values_indirectly(value_handle);
 
-  for (i = 0; i < (RE_MAX_MATCHES) && i < match_num; i++)
+  for (i = 0; i < (LOGMSG_MAX_MATCHES) && i < result->num_matches; i++)
     {
-      gint begin_index = matches[2 * i];
-      gint end_index = matches[2 * i + 1];
+      gint begin_index = result->matches[2 * i];
+      gint end_index = result->matches[2 * i + 1];
 
       if (begin_index < 0 || end_index < 0)
         continue;
 
-      if (indirect)
-        {
-          log_msg_set_match_indirect(msg, i, value_handle, 0, begin_index, end_index - begin_index);
-        }
-      else
-        {
-          log_msg_set_match(msg, i, &value[begin_index], end_index - begin_index);
-        }
+      log_matcher_pcre_re_feed_value(self, msg, log_msg_get_match_handle(i), result, begin_index, end_index);
     }
-}
-
-static inline void
-log_matcher_pcre_re_feed_value_by_name(LogMatcherPcreRe *s, LogMessage *msg, GString *formatted_name, gchar *tabptr,
-                                       const gchar *value, gint begin_index, gint end_index)
-{
-  if(s->nv_prefix != NULL)
+  if (log_msg_is_handle_match(result->source_handle) &&
+      log_msg_get_match_index(result->source_handle) >= result->num_matches)
     {
-      if (formatted_name->len > 0)
-        g_string_truncate(formatted_name, s->nv_prefix_len);
-      else
-        g_string_assign(formatted_name, s->nv_prefix);
-      g_string_append(formatted_name, tabptr + 2);
-
-      log_msg_set_value_by_name(msg, formatted_name->str, value + begin_index, end_index - begin_index);
-    }
-  else
-    {
-      log_msg_set_value_by_name(msg, tabptr + 2, value + begin_index, end_index - begin_index);
+      log_matcher_pcre_re_save_source_value_to_avoid_clobbering(result);
     }
+  log_msg_truncate_matches(msg, result->num_matches);
 }
 
 static void
-log_matcher_pcre_re_feed_named_substrings(LogMatcher *s, LogMessage *msg, int *matches, const gchar *value)
+log_matcher_pcre_re_feed_named_substrings(LogMatcherPcreRe *self, LogMessage *msg, LogMatcherPcreMatchResult *result)
 {
   gchar *name_table = NULL;
   gint i = 0;
   gint namecount = 0;
   gint name_entry_size = 0;
-  LogMatcherPcreRe *self = (LogMatcherPcreRe *) s;
 
   pcre_fullinfo(self->pattern, self->extra, PCRE_INFO_NAMECOUNT, &namecount);
   if (namecount > 0)
@@ -459,17 +474,24 @@ log_matcher_pcre_re_feed_named_substrings(LogMatcher *s, LogMessage *msg, int *m
          and the substring itself.
        */
       GString *formatted_name = scratch_buffers_alloc();
+      g_string_assign_len(formatted_name, self->nv_prefix, self->nv_prefix_len);
+
       tabptr = name_table;
       for (i = 0; i < namecount; i++, tabptr += name_entry_size)
         {
           int n = (tabptr[0] << 8) | tabptr[1];
-          gint begin_index = matches[2 * n];
-          gint end_index = matches[2 * n + 1];
+          gint begin_index = result->matches[2 * n];
+          gint end_index = result->matches[2 * n + 1];
+          const gchar *namedgroup_name = tabptr + 2;
 
           if (begin_index < 0 || end_index < 0)
             continue;
 
-          log_matcher_pcre_re_feed_value_by_name(self, msg, formatted_name, tabptr, value, begin_index, end_index);
+          g_string_truncate(formatted_name, self->nv_prefix_len);
+          g_string_append(formatted_name, namedgroup_name);
+
+          log_matcher_pcre_re_feed_value(self, msg, log_msg_get_value_handle(formatted_name->str),
+                                         result, begin_index, end_index);
         }
     }
 }
@@ -478,24 +500,27 @@ static gboolean
 log_matcher_pcre_re_match(LogMatcher *s, LogMessage *msg, gint value_handle, const gchar *value, gssize value_len)
 {
   LogMatcherPcreRe *self = (LogMatcherPcreRe *) s;
-  gint *matches;
-  gsize matches_size;
-  gint num_matches;
+  LogMatcherPcreMatchResult result;
   gint rc;
 
   if (value_len == -1)
     value_len = strlen(value);
 
-  if (pcre_fullinfo(self->pattern, self->extra, PCRE_INFO_CAPTURECOUNT, &num_matches) < 0)
+  if (pcre_fullinfo(self->pattern, self->extra, PCRE_INFO_CAPTURECOUNT, &result.num_matches) < 0)
     g_assert_not_reached();
-  if (num_matches > RE_MAX_MATCHES)
-    num_matches = RE_MAX_MATCHES;
+  if (result.num_matches > LOGMSG_MAX_MATCHES)
+    result.num_matches = LOGMSG_MAX_MATCHES;
 
-  matches_size = 3 * (num_matches + 1);
-  matches = g_alloca(matches_size * sizeof(gint));
+  gsize matches_size = 3 * (result.num_matches + 1);
+  result.matches = g_alloca(matches_size * sizeof(gint));
+  result.source_value = value;
+  result.source_value_len = value_len;
+  result.source_handle = value_handle;
 
   rc = pcre_exec(self->pattern, self->extra,
-                 value, value_len, 0, self->match_options, matches, matches_size);
+                 result.source_value, result.source_value_len,
+                 0, self->match_options,
+                 result.matches, matches_size);
   if (rc < 0)
     {
       switch (rc)
@@ -517,10 +542,11 @@ log_matcher_pcre_re_match(LogMatcher *s, LogMessage *msg, gint value_handle, con
     }
   else
     {
+      result.num_matches = rc;
       if ((s->flags & LMF_STORE_MATCHES))
         {
-          log_matcher_pcre_re_feed_backrefs(s, msg, value_handle, matches, rc, value);
-          log_matcher_pcre_re_feed_named_substrings(s, msg, matches, value);
+          log_matcher_pcre_re_feed_backrefs(self, msg, &result);
+          log_matcher_pcre_re_feed_named_substrings(self, msg, &result);
         }
     }
   return TRUE;
@@ -531,31 +557,34 @@ log_matcher_pcre_re_replace(LogMatcher *s, LogMessage *msg, gint value_handle, c
                             LogTemplate *replacement, gssize *new_length)
 {
   LogMatcherPcreRe *self = (LogMatcherPcreRe *) s;
+  LogMatcherPcreMatchResult result;
   GString *new_value = NULL;
-  gint *matches;
   gsize matches_size;
-  gint num_matches;
   gint rc;
   gint start_offset, last_offset;
   gint options;
   gboolean last_match_was_empty;
 
-  if (pcre_fullinfo(self->pattern, self->extra, PCRE_INFO_CAPTURECOUNT, &num_matches) < 0)
+  if (pcre_fullinfo(self->pattern, self->extra, PCRE_INFO_CAPTURECOUNT, &result.num_matches) < 0)
     g_assert_not_reached();
-  if (num_matches > RE_MAX_MATCHES)
-    num_matches = RE_MAX_MATCHES;
+  if (result.num_matches > LOGMSG_MAX_MATCHES)
+    result.num_matches = LOGMSG_MAX_MATCHES;
 
-  matches_size = 3 * (num_matches + 1);
-  matches = g_alloca(matches_size * sizeof(gint));
+  matches_size = 3 * (result.num_matches + 1);
+  result.matches = g_alloca(matches_size * sizeof(gint));
 
   /* we need zero initialized offsets for the last match as the
    * algorithm tries uses that as the base position */
 
-  matches[0] = matches[1] = matches[2] = 0;
+  result.matches[0] = result.matches[1] = result.matches[2] = 0;
 
   if (value_len == -1)
     value_len = strlen(value);
 
+  result.source_value = value;
+  result.source_value_len = value_len;
+  result.source_handle = value_handle;
+
   last_offset = start_offset = 0;
   last_match_was_empty = FALSE;
   do
@@ -596,8 +625,8 @@ log_matcher_pcre_re_replace(LogMatcher *s, LogMessage *msg, gint value_handle, c
         }
 
       rc = pcre_exec(self->pattern, self->extra,
-                     value, value_len,
-                     start_offset, (self->match_options | options), matches, matches_size);
+                     result.source_value, result.source_value_len,
+                     start_offset, (self->match_options | options), result.matches, matches_size);
       if (rc < 0 && rc != PCRE_ERROR_NOMATCH)
         {
           msg_error("Error while matching regexp",
@@ -625,31 +654,32 @@ log_matcher_pcre_re_replace(LogMatcher *s, LogMessage *msg, gint value_handle, c
       else
         {
           /* if the output array was too small, truncate the number of
-             captures to RE_MAX_MATCHES */
+             captures to LOGMSG_MAX_MATCHES */
 
           if (rc == 0)
             rc = matches_size / 3;
 
-          log_matcher_pcre_re_feed_backrefs(s, msg, value_handle, matches, rc, value);
-          log_matcher_pcre_re_feed_named_substrings(s, msg, matches, value);
+          result.num_matches = rc;
+          log_matcher_pcre_re_feed_backrefs(self, msg, &result);
+          log_matcher_pcre_re_feed_named_substrings(self, msg, &result);
 
           if (!new_value)
-            new_value = g_string_sized_new(value_len);
+            new_value = g_string_sized_new(result.source_value_len);
           /* append non-matching portion */
-          g_string_append_len(new_value, &value[last_offset], matches[0] - last_offset);
+          g_string_append_len(new_value, &result.source_value[last_offset], result.matches[0] - last_offset);
           /* replacement */
           log_template_append_format(replacement, msg, &DEFAULT_TEMPLATE_EVAL_OPTIONS, new_value);
 
-          last_match_was_empty = (matches[0] == matches[1]);
-          start_offset = last_offset = matches[1];
+          last_match_was_empty = (result.matches[0] == result.matches[1]);
+          start_offset = last_offset = result.matches[1];
         }
     }
-  while (self->super.flags & LMF_GLOBAL && start_offset < value_len);
+  while (self->super.flags & LMF_GLOBAL && start_offset < result.source_value_len);
 
   if (new_value)
     {
       /* append the last literal */
-      g_string_append_len(new_value, &value[last_offset], value_len - last_offset);
+      g_string_append_len(new_value, &result.source_value[last_offset], result.source_value_len - last_offset);
       if (new_length)
         *new_length = new_value->len;
       return g_string_free(new_value, FALSE);
@@ -702,6 +732,57 @@ log_matcher_pcre_set_nv_prefix(LogMatcher *s, const gchar *prefix)
 
 typedef LogMatcher *(*LogMatcherConstructFunc)(const LogMatcherOptions *options);
 
+gboolean
+log_matcher_match_value(LogMatcher *s, LogMessage *msg, gint value_handle)
+{
+  NVTable *payload = nv_table_ref(msg->payload);
+  gssize value_len;
+  const gchar *value = log_msg_get_value(msg, value_handle, &value_len);
+
+  APPEND_ZERO(value, value, value_len);
+
+  gboolean result = log_matcher_match(s, msg, value_handle, value, value_len);
+  nv_table_unref(payload);
+  return result;
+}
+
+gboolean
+log_matcher_match_buffer(LogMatcher *s, LogMessage *msg, const gchar *value, gssize value_len)
+{
+  return log_matcher_match(s, msg, LM_V_NONE, value, value_len);
+}
+
+gboolean
+log_matcher_match_template(LogMatcher *s, LogMessage *msg, LogTemplate *template, LogTemplateEvalOptions *options)
+{
+  gboolean result;
+
+  if (log_template_is_literal_string(template))
+    {
+      gssize len;
+      const gchar *value = log_template_get_literal_value(template, &len);
+
+      result = log_matcher_match_buffer(s, msg, value, len);
+
+    }
+  else if (log_template_is_trivial(template))
+    {
+      NVHandle handle = log_template_get_trivial_value_handle(template);
+
+      g_assert(handle != LM_V_NONE);
+      result = log_matcher_match_value(s, msg, handle);
+    }
+  else
+    {
+      GString *buffer = scratch_buffers_alloc();
+
+      log_template_format(template, msg, options, buffer);
+      result = log_matcher_match_buffer(s, msg, buffer->str, buffer->len);
+    }
+  return result;
+}
+
+
 struct
 {
   const gchar *name;
