@@ -44,7 +44,7 @@ assert_nvtable(NVTable *tab, NVHandle handle, gchar *expected_value, gssize expe
   const gchar *value;
   gssize length;
 
-  value = nv_table_get_value(tab, handle, &length);
+  value = nv_table_get_value(tab, handle, &length, NULL);
 
   cr_assert_eq(length, expected_length,
                "NVTable value mismatch, value=%.*s, expected=%.*s\n",
@@ -189,14 +189,14 @@ Test(nvtable, test_nvtable_direct)
 
       /* one that fits */
       tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, 0, NULL);
       cr_assert(success);
       assert_nvtable(tab, handle, value, 128);
       nv_table_unref(tab);
 
       /* one that is too large */
       tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 512, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 512, 0, NULL);
       cr_assert_not(success);
       nv_table_unref(tab);
 
@@ -206,11 +206,11 @@ Test(nvtable, test_nvtable_direct)
 
       /* one that fits, but realloced size wouldn't fit */
       tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, 0, NULL);
       cr_assert(success);
       used = tab->used;
 
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, 0, NULL);
       cr_assert(success);
       cr_assert_eq(tab->used, used);
       assert_nvtable(tab, handle, value, 64);
@@ -218,11 +218,11 @@ Test(nvtable, test_nvtable_direct)
 
       /* one that is too large for the given entry, but still fits in the NVTable */
       tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, 0, NULL);
       cr_assert(success);
       used = tab->used;
 
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, 0, NULL);
       cr_assert(success);
       cr_assert_gt(tab->used, used);
       assert_nvtable(tab, handle, value, 128);
@@ -230,10 +230,10 @@ Test(nvtable, test_nvtable_direct)
 
       /* one that is too large for the given entry, and also for the NVTable */
       tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, 0, NULL);
       cr_assert(success);
 
-      success = nv_table_add_value(tab, handle, name, strlen(name), value, 512, NULL);
+      success = nv_table_add_value(tab, handle, name, strlen(name), value, 512, 0, NULL);
       cr_assert_not(success);
       assert_nvtable(tab, handle, value, 64);
       nv_table_unref(tab);
@@ -248,18 +248,18 @@ Test(nvtable, test_nvtable_direct)
 
           /* setup code: add static and a dynamic-indirect entry */
           tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
           cr_assert(success);
           success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                                 &(NVReferencedSlice)
           {
-            STATIC_HANDLE, 1, 126, 0
-          }, NULL);
+            STATIC_HANDLE, 1, 126
+          }, 0, NULL);
           cr_assert(success);
           used = tab->used;
 
           /* store a direct entry over the indirect one */
-          success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, NULL);
+          success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, 0, NULL);
           cr_assert(success);
           cr_assert_eq(tab->used, used);
           assert_nvtable(tab, STATIC_HANDLE, value, 128);
@@ -269,19 +269,19 @@ Test(nvtable, test_nvtable_direct)
 
           /* setup code: add static and a dynamic-indirect entry */
           tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 64, NULL);
+          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 64, 0, NULL);
           cr_assert(success);
           success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                                 &(NVReferencedSlice)
           {
-            STATIC_HANDLE, 1, 63, 0
-          }, NULL);
+            STATIC_HANDLE, 1, 63
+          }, 0, NULL);
           cr_assert(success);
 
           used = tab->used;
 
           /* store a direct entry over the indirect one, we don't fit in the allocated space */
-          success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, NULL);
+          success = nv_table_add_value(tab, handle, name, strlen(name), value, 128, 0, NULL);
           cr_assert(success);
           cr_assert_gt(tab->used, used);
           assert_nvtable(tab, STATIC_HANDLE, value, 64);
@@ -291,18 +291,18 @@ Test(nvtable, test_nvtable_direct)
 
           /* setup code: add static and a dynamic-indirect entry */
           tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 64, NULL);
+          success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 64, 0, NULL);
           cr_assert(success);
           success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                                 &(NVReferencedSlice)
           {
-            STATIC_HANDLE, 1, 62, 0
-          }, NULL);
+            STATIC_HANDLE, 1, 62
+          }, 0, NULL);
           cr_assert(success);
           used = tab->used;
 
           /* store a direct entry over the indirect one, we don't fit in the allocated space */
-          success = nv_table_add_value(tab, handle, name, strlen(name), value, 256, NULL);
+          success = nv_table_add_value(tab, handle, name, strlen(name), value, 256, 0, NULL);
           cr_assert_not(success);
           assert_nvtable(tab, STATIC_HANDLE, value, 64);
           assert_nvtable(tab, handle, value + 1, 62);
@@ -372,14 +372,14 @@ Test(nvtable, test_nvtable_indirect)
 
   /* one that fits */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
   assert_nvtable(tab, handle, value + 1, 126);
@@ -391,14 +391,14 @@ Test(nvtable, test_nvtable_indirect)
      to make it possible to store one direct entry */
 
   tab = nv_table_new(STATIC_VALUES, 0, 138+3);  // direct: +3
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert_not(success);
 
   nv_table_unref(tab);
@@ -408,21 +408,21 @@ Test(nvtable, test_nvtable_indirect)
   /*************************************************************/
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 62, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 62
+  }, 0, NULL);
 
   cr_assert(success);
   cr_assert_eq(used, tab->used);
@@ -439,17 +439,17 @@ Test(nvtable, test_nvtable_indirect)
 
   /* setup code: add static and a dynamic-direct entry */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   cr_assert_eq(tab->used, used);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
@@ -461,17 +461,17 @@ Test(nvtable, test_nvtable_indirect)
 
   /* setup code: add static and a dynamic-direct entry */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   cr_assert_gt(tab->used, used);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
@@ -483,16 +483,16 @@ Test(nvtable, test_nvtable_indirect)
 
   /* setup code: add static and a dynamic-direct entry */
   tab = nv_table_new(STATIC_VALUES, 1, 154+3+4);  // direct: +3, indirect: +4
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 1, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert_not(success);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
   assert_nvtable(tab, handle, value, 1);
@@ -512,20 +512,20 @@ Test(nvtable, test_nvtable_indirect)
 
   /* one that fits */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 122, 0
-  }, NULL);
+    DYN_HANDLE, 1, 122
+  }, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
   assert_nvtable(tab, DYN_HANDLE, value + 1, 126);
@@ -534,20 +534,20 @@ Test(nvtable, test_nvtable_indirect)
 
   /* one that is too large */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 122, 0
-  }, NULL);
+    DYN_HANDLE, 1, 122
+  }, 0, NULL);
   cr_assert_not(success);
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
   assert_nvtable(tab, DYN_HANDLE, value + 1, 126);
@@ -561,27 +561,27 @@ Test(nvtable, test_nvtable_indirect)
 
   /* we fit to the space of the old */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 1, 0
-  }, NULL);
+    DYN_HANDLE, 1, 1
+  }, 0, NULL);
   cr_assert(success);
 
   cr_assert_eq(tab->used, used);
@@ -593,27 +593,27 @@ Test(nvtable, test_nvtable_indirect)
   /* the new entry will not fit to the space allocated to the old */
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 16, 0
-  }, NULL);
+    DYN_HANDLE, 1, 16
+  }, 0, NULL);
   cr_assert(success);
 
   cr_assert_gt(tab->used, used);
@@ -625,27 +625,27 @@ Test(nvtable, test_nvtable_indirect)
   /* one that is too large */
 
   tab = nv_table_new(STATIC_VALUES, 4, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 124, 0
-  }, NULL);
+    DYN_HANDLE, 1, 124
+  }, 0, NULL);
   cr_assert_not(success);
 
   cr_assert_eq(tab->used, used);
@@ -661,23 +661,23 @@ Test(nvtable, test_nvtable_indirect)
 
   /* we fit to the space of the old */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 64, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 16, 0
-  }, NULL);
+    DYN_HANDLE, 1, 16
+  }, 0, NULL);
   cr_assert(success);
 
   cr_assert_eq(tab->used, used);
@@ -690,23 +690,23 @@ Test(nvtable, test_nvtable_indirect)
   /* the new entry will not fit to the space allocated to the old */
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 16, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 16, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 32, 0
-  }, NULL);
+    DYN_HANDLE, 1, 32
+  }, 0, NULL);
   cr_assert(success);
 
   cr_assert_gt(tab->used, used);
@@ -718,23 +718,23 @@ Test(nvtable, test_nvtable_indirect)
   /* one that is too large */
 
   tab = nv_table_new(STATIC_VALUES, 4, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
   success = nv_table_add_value_indirect(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, handle, name, strlen(name), value, 16, NULL);
+  success = nv_table_add_value(tab, handle, name, strlen(name), value, 16, 0, NULL);
   cr_assert(success);
   used = tab->used;
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    DYN_HANDLE, 1, 124, 0
-  }, NULL);
+    DYN_HANDLE, 1, 124
+  }, 0, NULL);
   cr_assert_not(success);
 
   cr_assert_eq(tab->used, used);
@@ -753,8 +753,8 @@ Test(nvtable, test_nvtable_indirect)
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
   cr_assert_eq(used, tab->used);
   assert_nvtable(tab, STATIC_HANDLE, "", 0);
@@ -790,16 +790,16 @@ Test(nvtable, test_nvtable_others)
 
   /* one that fits */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 256);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value + 32, 32, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value + 32, 32, 0, NULL);
   cr_assert(success);
 
   assert_nvtable(tab, STATIC_HANDLE, value + 32, 32);
@@ -808,16 +808,16 @@ Test(nvtable, test_nvtable_others)
 
   /* one that doesn't fit */
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 192);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value, 128, 0, NULL);
   cr_assert(success);
 
   success = nv_table_add_value_indirect(tab, handle, name, strlen(name),
                                         &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 126, 0
-  }, NULL);
+    STATIC_HANDLE, 1, 126
+  }, 0, NULL);
   cr_assert(success);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value + 32, 32, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, 4, value + 32, 32, 0, NULL);
   cr_assert_not(success);
 
   assert_nvtable(tab, STATIC_HANDLE, value, 128);
@@ -849,7 +849,7 @@ Test(nvtable, test_nvtable_lookup)
             }
           while (handle == 0);
           g_snprintf(name, sizeof(name), "VAL%d", handle);
-          success = nv_table_add_value(tab, handle, name, strlen(name), name, strlen(name), NULL);
+          success = nv_table_add_value(tab, handle, name, strlen(name), name, strlen(name), 0, NULL);
           g_assert(success);
           handles[i] = handle;
         }
@@ -873,7 +873,7 @@ Test(nvtable, test_nvtable_clone_grows_the_cloned_structure)
   gboolean success;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 64);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, "value", 5);
 
@@ -890,7 +890,7 @@ Test(nvtable, test_nvtable_clone_cannot_grow_nvtable_larger_than_nvtable_max_byt
   gboolean success;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, NV_TABLE_MAX_BYTES - 1024);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, "value", 5);
 
@@ -909,7 +909,7 @@ Test(nvtable, test_nvtable_realloc_doubles_nvtable_size)
   gsize old_size;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, "value", 5);
 
@@ -930,7 +930,7 @@ Test(nvtable, test_nvtable_realloc_sets_size_to_nv_table_max_bytes_at_most)
   gsize old_size;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, NV_TABLE_MAX_BYTES - 1024);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, "value", 5);
 
@@ -950,7 +950,7 @@ Test(nvtable, test_nvtable_realloc_fails_if_size_is_at_maximum)
   gboolean success;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, NV_TABLE_MAX_BYTES);
-  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab, STATIC_HANDLE, "value", 5);
 
@@ -970,7 +970,7 @@ Test(nvtable, test_nvtable_realloc_leaves_original_intact_if_there_are_multiple_
   tab_ref1 = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
   tab_ref2 = nv_table_ref(tab_ref1);
 
-  success = nv_table_add_value(tab_ref1, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, NULL);
+  success = nv_table_add_value(tab_ref1, STATIC_HANDLE, STATIC_NAME, strlen(STATIC_NAME), "value", 5, 0, NULL);
   cr_assert(success);
   assert_nvtable(tab_ref1, STATIC_HANDLE, "value", 5);
   assert_nvtable(tab_ref2, STATIC_HANDLE, "value", 5);
@@ -995,27 +995,26 @@ Test(nvtable, test_nvtable_unset_values)
   gboolean success;
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
-  value = nv_table_get_value(tab, DYN_HANDLE, &size);
-  cr_assert_not_null(value);
-  cr_assert_eq(value[0], 0);
-  cr_assert_eq(size, 0);
-
-  size = 1;
-  value = nv_table_get_value_if_set(tab, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
   cr_assert_null(value);
   cr_assert_eq(size, 0);
 
-  success = nv_table_add_value(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "foo", 3, NULL);
+  size = 1;
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
+  cr_assert_null(value);
+  cr_assert_eq(size, 0);
+
+  success = nv_table_add_value(tab, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "foo", 3, 0, NULL);
   cr_assert(success);
   size = 1;
-  value = nv_table_get_value_if_set(tab, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert_arr_eq(value, "foo", 3);
   cr_assert_eq(size, 3);
 
   nv_table_unset_value(tab, DYN_HANDLE);
   size = 1;
-  value = nv_table_get_value_if_set(tab, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
   cr_assert_null(value);
   cr_assert_eq(size, 0);
 
@@ -1030,22 +1029,21 @@ Test(nvtable, test_nvtable_unset_copies_indirect_references)
   const gchar *indirect_nv_name = "indirect-name";
 
   tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
-  nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, NULL);
+  nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, 0, NULL);
   nv_table_add_value_indirect(tab, DYN_HANDLE, indirect_nv_name, strlen(indirect_nv_name),
                               &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 5, 0
-  },
-  NULL);
+    STATIC_HANDLE, 1, 5
+  }, 0, NULL);
 
-  value = nv_table_get_value(tab, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert(strncmp(value, "tatic", 5) == 0);
   cr_assert_eq(size, 5);
 
   nv_table_unset_value(tab, STATIC_HANDLE);
 
-  value = nv_table_get_value(tab, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert(strncmp(value, "tatic", 5) == 0);
   cr_assert_eq(size, 5);
@@ -1053,6 +1051,45 @@ Test(nvtable, test_nvtable_unset_copies_indirect_references)
   nv_table_unref(tab);
 }
 
+Test(nvtable, test_nvtable_indirect_references_unset_and_then_set_again_are_present)
+{
+  NVTable *tab;
+  gssize size = 9999;
+  const gchar *value;
+  const gchar *indirect_nv_name = "indirect-name";
+
+  tab = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
+  nv_table_add_value(tab, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, 0, NULL);
+  nv_table_add_value_indirect(tab, DYN_HANDLE, indirect_nv_name, strlen(indirect_nv_name),
+                              &(NVReferencedSlice)
+  {
+    STATIC_HANDLE, 1, 5
+  }, 0, NULL);
+
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
+  cr_assert_not_null(value);
+  cr_assert(strncmp(value, "tatic", 5) == 0);
+  cr_assert_eq(size, 5);
+
+  nv_table_unset_value(tab, DYN_HANDLE);
+
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
+  cr_assert_null(value);
+
+  nv_table_add_value_indirect(tab, DYN_HANDLE, indirect_nv_name, strlen(indirect_nv_name),
+                              &(NVReferencedSlice)
+  {
+    STATIC_HANDLE, 2, 4
+  }, 0, NULL);
+
+  value = nv_table_get_value(tab, DYN_HANDLE, &size, NULL);
+  cr_assert_not_null(value);
+  cr_assert(strncmp(value, "atic", 4) == 0);
+  cr_assert_eq(size, 4);
+
+  nv_table_unref(tab);
+}
+
 Test(nvtable, test_nvtable_compact_copies_name_value_pairs)
 {
   NVTable *tab1, *tab2;
@@ -1061,29 +1098,28 @@ Test(nvtable, test_nvtable_compact_copies_name_value_pairs)
   const gchar *indirect_nv_name = "indirect-name";
 
   tab1 = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
-  nv_table_add_value(tab1, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, NULL);
-  nv_table_add_value(tab1, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "dyn-foo", 7, NULL);
+  nv_table_add_value(tab1, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, 0, NULL);
+  nv_table_add_value(tab1, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "dyn-foo", 7, 0, NULL);
   nv_table_add_value_indirect(tab1, DYN_HANDLE+1, indirect_nv_name, strlen(indirect_nv_name),
                               &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 5, 0
-  },
-  NULL);
+    STATIC_HANDLE, 1, 5
+  }, 0, NULL);
 
   tab2 = nv_table_compact(tab1);
   nv_table_unref(tab1);
 
-  value = nv_table_get_value(tab2, DYN_HANDLE, &size);
+  value = nv_table_get_value(tab2, DYN_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert_str_eq(value, "dyn-foo");
   cr_assert_eq(size, 7);
 
-  value = nv_table_get_value(tab2, STATIC_HANDLE, &size);
+  value = nv_table_get_value(tab2, STATIC_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert_str_eq(value, "static-foo");
   cr_assert_eq(size, 10);
 
-  value = nv_table_get_value(tab2, DYN_HANDLE+1, &size);
+  value = nv_table_get_value(tab2, DYN_HANDLE+1, &size, NULL);
   cr_assert_not_null(value);
   cr_assert(strncmp(value, "tatic", size) == 0);
   cr_assert_eq(size, 5);
@@ -1099,14 +1135,13 @@ Test(nvtable, test_nvtable_compact_skips_unset_values)
   const gchar *indirect_nv_name = "indirect-name";
 
   tab1 = nv_table_new(STATIC_VALUES, STATIC_VALUES, 1024);
-  nv_table_add_value(tab1, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, NULL);
-  nv_table_add_value(tab1, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "dyn-foo", 7, NULL);
+  nv_table_add_value(tab1, STATIC_HANDLE, STATIC_NAME, strlen(DYN_NAME), "static-foo", 10, 0, NULL);
+  nv_table_add_value(tab1, DYN_HANDLE, DYN_NAME, strlen(DYN_NAME), "dyn-foo", 7, 0, NULL);
   nv_table_add_value_indirect(tab1, DYN_HANDLE+1, indirect_nv_name, strlen(indirect_nv_name),
                               &(NVReferencedSlice)
   {
-    STATIC_HANDLE, 1, 5, 0
-  },
-  NULL);
+    STATIC_HANDLE, 1, 5
+  }, 0, NULL);
   nv_table_unset_value(tab1, DYN_HANDLE);
 
   /* this should get rid off the unset value, thus used should be smaller */
@@ -1115,17 +1150,16 @@ Test(nvtable, test_nvtable_compact_skips_unset_values)
 
   nv_table_unref(tab1);
 
-  value = nv_table_get_value(tab2, DYN_HANDLE, &size);
-  cr_assert_not_null(value);
-  cr_assert_str_eq(value, "");
+  value = nv_table_get_value(tab2, DYN_HANDLE, &size, NULL);
+  cr_assert_null(value);
   cr_assert_eq(size, 0);
 
-  value = nv_table_get_value(tab2, STATIC_HANDLE, &size);
+  value = nv_table_get_value(tab2, STATIC_HANDLE, &size, NULL);
   cr_assert_not_null(value);
   cr_assert_str_eq(value, "static-foo");
   cr_assert_eq(size, 10);
 
-  value = nv_table_get_value(tab2, DYN_HANDLE+1, &size);
+  value = nv_table_get_value(tab2, DYN_HANDLE+1, &size, NULL);
   cr_assert_not_null(value);
   cr_assert(strncmp(value, "tatic", size) == 0);
   cr_assert_eq(size, 5);
