@@ -22,14 +22,17 @@
  *
  */
 
-#include "mock-transport.h"
-#include "proto_lib.h"
-#include "msg_parse_lib.h"
+#include <criterion/criterion.h>
+#include "libtest/mock-transport.h"
+#include "libtest/proto_lib.h"
+#include "libtest/msg_parse_lib.h"
+#include "libtest/grab-logging.h"
+
 #include "logproto/logproto-text-server.h"
+#include "ack-tracker/ack_tracker_factory.h"
 
 #include <errno.h>
 
-#include <criterion/criterion.h>
 
 static gint accumulate_seq;
 
@@ -41,6 +44,14 @@ construct_test_proto(LogTransport *transport)
   return log_proto_text_server_new(transport, get_inited_proto_server_options());
 }
 
+LogProtoServer *
+construct_test_proto_with_nuls(LogTransport *transport)
+{
+  proto_server_options.max_msg_size = 32;
+
+  return log_proto_text_with_nuls_server_new(transport, get_inited_proto_server_options());
+}
+
 LogProtoServer *
 construct_test_proto_with_accumulator(gint (*accumulator)(LogProtoTextServer *, const guchar *, gsize, gssize),
                                       LogTransport *transport)
@@ -141,6 +152,24 @@ Test(log_proto, test_log_proto_text_server_no_eol_before_eof)
 
 }
 
+Test(log_proto, test_log_proto_text_with_embedded_nuls)
+{
+  LogProtoServer *proto;
+
+  proto = construct_test_proto_with_nuls(
+            log_transport_mock_stream_new(
+              /* no eol before EOF */
+              "01234567\n", -1,
+              "alma\x00korte\n", 11,
+
+              LTM_EOF));
+
+  assert_proto_server_fetch(proto, "01234567", -1);
+  assert_proto_server_fetch(proto, "alma\x00korte", 10);
+  assert_proto_server_fetch_failure(proto, LPS_EOF, NULL);
+  log_proto_server_free(proto);
+}
+
 Test(log_proto, test_log_proto_text_server_eol_before_eof)
 {
   LogProtoServer *proto;
@@ -192,6 +221,22 @@ Test(log_proto, test_log_proto_text_server_partial_chars_before_eof)
   log_proto_server_free(proto);
 }
 
+Test(log_proto, test_log_proto_text_server_invalid_char_with_encoding)
+{
+  LogProtoServer *proto;
+
+  log_proto_server_options_set_encoding(&proto_server_options, "GB18030");
+  proto = construct_test_proto(
+            log_transport_mock_stream_new(
+              "foo" "\x80" "bar" "\x80" "baz", -1,
+              LTM_EOF));
+
+  cr_assert(log_proto_server_validate_options(proto),
+            "validate_options() returned failure but it should have succeeded");
+  assert_proto_server_fetch(proto, "foobarbaz", -1);
+  log_proto_server_free(proto);
+}
+
 Test(log_proto, test_log_proto_text_server_not_fixed_encoding)
 {
   LogProtoServer *proto;
@@ -266,8 +311,7 @@ Test(log_proto, test_log_proto_text_server_invalid_encoding)
 
   start_grabbing_messages();
   success = log_proto_server_validate_options(proto);
-  assert_grabbed_messages_contain("Unknown character set name specified; encoding='never-ever-is-going-to-be-such-an-encoding'",
-                                  "message about unknown charset missing");
+  assert_grabbed_log_contains("Unknown character set name specified; encoding='never-ever-is-going-to-be-such-an-encoding'");
   cr_assert_not(success, "validate_options() returned success but it should have failed");
   log_proto_server_free(proto);
 }
@@ -589,3 +633,33 @@ Test(log_proto, test_log_proto_text_server_io_eagain)
 
   log_proto_server_free(proto);
 }
+
+Test(log_proto, buffer_split_with_encoding_and_position_tracking)
+{
+  GString *data = g_string_new("Lorem ipsum\xe2\x98\x83lor sit amet, consectetur adipiscing elit\n");
+  GString *data_smaller = g_string_new("muspi merol\n");
+  gsize bytes_should_not_fit = 5;
+
+  proto_server_options.max_msg_size = data->len + data_smaller->len - bytes_should_not_fit;
+  proto_server_options.max_buffer_size = proto_server_options.max_msg_size;
+  log_proto_server_options_set_encoding(&proto_server_options, "utf-8");
+  log_proto_server_options_set_ack_tracker_factory(&proto_server_options, consecutive_ack_tracker_factory_new());
+
+  gchar *full_payload = g_strconcat(data_smaller->str, data->str, NULL);
+  LogTransportMock *transport = (LogTransportMock *) log_transport_mock_records_new(full_payload, -1, LTM_EOF);
+
+  LogProtoServer *proto = log_proto_text_server_new((LogTransport *) transport, get_inited_proto_server_options());
+
+  start_grabbing_messages();
+  assert_proto_server_fetch(proto, data_smaller->str, data_smaller->len - 1);
+  assert_proto_server_fetch(proto, data->str, data->len - 1);
+  stop_grabbing_messages();
+  cr_assert_not(find_grabbed_message("Internal error"));
+
+  assert_proto_server_fetch_failure(proto, LPS_EOF, NULL);
+
+  log_proto_server_free(proto);
+  g_free(full_payload);
+  g_string_free(data_smaller, TRUE);
+  g_string_free(data, TRUE);
+}
