@@ -23,10 +23,13 @@
  */
 
 #include "stats/stats-cluster-logpipe.h"
+#include "stats/stats-cluster-single.h"
+#include "stats/aggregator/stats-aggregator-registry.h"
 #include "logthrdestdrv.h"
 #include "seqnum.h"
 #include "scratch-buffers.h"
 #include "timeutils/misc.h"
+#include "mainloop-threaded-worker.h"
 
 #define MAX_RETRIES_ON_ERROR_DEFAULT 3
 #define MAX_RETRIES_BEFORE_SUSPEND_DEFAULT 3
@@ -652,37 +655,6 @@ _init_watches(LogThreadedDestWorker *self)
   self->do_work.handler = _perform_work;
 }
 
-static void
-_signal_startup_finished(LogThreadedDestWorker *self, gboolean thread_failure)
-{
-  g_mutex_lock(self->owner->lock);
-  self->startup_finished = TRUE;
-  self->startup_failure |= thread_failure;
-  g_cond_signal(self->started_up);
-  g_mutex_unlock(self->owner->lock);
-}
-
-static void
-_signal_startup_success(LogThreadedDestWorker *self)
-{
-  _signal_startup_finished(self, FALSE);
-}
-
-static void
-_signal_startup_failure(LogThreadedDestWorker *self)
-{
-  _signal_startup_finished(self, TRUE);
-}
-
-static void
-_wait_for_startup_finished(LogThreadedDestWorker *self)
-{
-  g_mutex_lock(self->owner->lock);
-  while (!self->startup_finished)
-    g_cond_wait(self->started_up, self->owner->lock);
-  g_mutex_unlock(self->owner->lock);
-}
-
 static void
 _register_worker_stats(LogThreadedDestWorker *self)
 {
@@ -720,26 +692,38 @@ _perform_final_flush(LogThreadedDestWorker *self)
   log_queue_rewind_backlog_all(self->queue);
 }
 
-static void
-_worker_thread(gpointer arg)
+static gboolean
+_worker_thread_init(MainLoopThreadedWorker *s)
 {
-  LogThreadedDestWorker *self = (LogThreadedDestWorker *) arg;
+  LogThreadedDestWorker *self = (LogThreadedDestWorker *) s->data;
 
-  iv_init();
+  iv_event_register(&self->wake_up_event);
+  iv_event_register(&self->shutdown_event);
+
+  return log_threaded_dest_worker_init(self);
+}
+
+static void
+_worker_thread_deinit(MainLoopThreadedWorker *s)
+{
+  LogThreadedDestWorker *self = (LogThreadedDestWorker *) s->data;
+
+  log_threaded_dest_worker_deinit(self);
+
+  iv_event_unregister(&self->wake_up_event);
+  iv_event_unregister(&self->shutdown_event);
+}
+
+static void
+_worker_thread(MainLoopThreadedWorker *s)
+{
+  LogThreadedDestWorker *self = (LogThreadedDestWorker *) s->data;
 
   msg_debug("Dedicated worker thread started",
             evt_tag_int("worker_index", self->worker_index),
             evt_tag_str("driver", self->owner->super.super.id),
             log_expr_node_location_tag(self->owner->super.super.super.expr_node));
 
-  iv_event_register(&self->wake_up_event);
-  iv_event_register(&self->shutdown_event);
-
-  if (!log_threaded_dest_worker_thread_init(self))
-    goto error;
-
-  _signal_startup_success(self);
-
   /* if we have anything on the backlog, that was a partial, potentially
    * not-flushed batch.  Rewind it, so we start with that */
 
@@ -752,21 +736,35 @@ _worker_thread(gpointer arg)
 
   _disconnect(self);
 
-  log_threaded_dest_worker_thread_deinit(self);
-
   msg_debug("Dedicated worker thread finished",
             evt_tag_int("worker_index", self->worker_index),
             evt_tag_str("driver", self->owner->super.super.id),
             log_expr_node_location_tag(self->owner->super.super.super.expr_node));
 
-  goto ok;
+}
 
-error:
-  _signal_startup_failure(self);
-ok:
-  iv_event_unregister(&self->wake_up_event);
-  iv_event_unregister(&self->shutdown_event);
-  iv_deinit();
+static void
+_request_worker_exit(MainLoopThreadedWorker *s)
+{
+  LogThreadedDestWorker *self = (LogThreadedDestWorker *) s->data;
+
+  msg_debug("Shutting down dedicated worker thread",
+            evt_tag_int("worker_index", self->worker_index),
+            evt_tag_str("driver", self->owner->super.super.id),
+            log_expr_node_location_tag(self->owner->super.super.super.expr_node));
+  self->owner->under_termination = TRUE;
+  iv_event_post(&self->shutdown_event);
+}
+
+static gboolean
+log_threaded_dest_worker_start(LogThreadedDestWorker *self)
+{
+  msg_debug("Starting dedicated worker thread",
+            evt_tag_int("worker_index", self->worker_index),
+            evt_tag_str("driver", self->owner->super.super.id),
+            log_expr_node_location_tag(self->owner->super.super.super.expr_node));
+
+  return main_loop_threaded_worker_start(&self->thread);
 }
 
 static gboolean
@@ -804,19 +802,23 @@ log_threaded_dest_worker_deinit_method(LogThreadedDestWorker *self)
 void
 log_threaded_dest_worker_free_method(LogThreadedDestWorker *self)
 {
-  g_cond_free(self->started_up);
+  main_loop_threaded_worker_clear(&self->thread);
 }
 
 void
 log_threaded_dest_worker_init_instance(LogThreadedDestWorker *self, LogThreadedDestDriver *owner, gint worker_index)
 {
+  main_loop_threaded_worker_init(&self->thread, MLW_THREADED_OUTPUT_WORKER, self);
+  self->thread.thread_init = _worker_thread_init;
+  self->thread.thread_deinit = _worker_thread_deinit;
+  self->thread.run = _worker_thread;
+  self->thread.request_exit = _request_worker_exit;
   self->worker_index = worker_index;
-  self->thread_init = log_threaded_dest_worker_init_method;
-  self->thread_deinit = log_threaded_dest_worker_deinit_method;
+  self->init = log_threaded_dest_worker_init_method;
+  self->deinit = log_threaded_dest_worker_deinit_method;
   self->free_fn = log_threaded_dest_worker_free_method;
   self->owner = owner;
   self->time_reopen = -1;
-  self->started_up = g_cond_new();
   _init_watches(self);
 }
 
@@ -841,7 +843,7 @@ log_threaded_dest_driver_set_num_workers(LogDriver *s, gint num_workers)
 /* compatibility bridge between LogThreadedDestWorker */
 
 static gboolean
-_compat_thread_init(LogThreadedDestWorker *self)
+_compat_init(LogThreadedDestWorker *self)
 {
   if (!log_threaded_dest_worker_init_method(self))
     return FALSE;
@@ -853,7 +855,7 @@ _compat_thread_init(LogThreadedDestWorker *self)
 }
 
 static void
-_compat_thread_deinit(LogThreadedDestWorker *self)
+_compat_deinit(LogThreadedDestWorker *self)
 {
   if (self->owner->worker.thread_deinit)
     self->owner->worker.thread_deinit(self->owner);
@@ -892,8 +894,8 @@ _compat_flush(LogThreadedDestWorker *self, LogThreadedFlushMode mode)
 static void
 _init_worker_compat_layer(LogThreadedDestWorker *self)
 {
-  self->thread_init = _compat_thread_init;
-  self->thread_deinit = _compat_thread_deinit;
+  self->init = _compat_init;
+  self->deinit = _compat_deinit;
   self->connect = _compat_connect;
   self->disconnect = _compat_disconnect;
   self->insert = _compat_insert;
@@ -922,33 +924,6 @@ _construct_worker(LogThreadedDestDriver *self, gint worker_index)
   return self->worker.construct(self, worker_index);
 }
 
-static void
-_request_worker_exit(gpointer s)
-{
-  LogThreadedDestWorker *self = (LogThreadedDestWorker *) s;
-
-  msg_debug("Shutting down dedicated worker thread",
-            evt_tag_int("worker_index", self->worker_index),
-            evt_tag_str("driver", self->owner->super.super.id),
-            log_expr_node_location_tag(self->owner->super.super.super.expr_node));
-  self->owner->under_termination = TRUE;
-  iv_event_post(&self->shutdown_event);
-}
-
-static gboolean
-_start_worker_thread(LogThreadedDestWorker *self)
-{
-  msg_debug("Starting dedicated worker thread",
-            evt_tag_int("worker_index", self->worker_index),
-            evt_tag_str("driver", self->owner->super.super.id),
-            log_expr_node_location_tag(self->owner->super.super.super.expr_node));
-
-  main_loop_create_worker_thread(_worker_thread,
-                                 _request_worker_exit,
-                                 self, &self->owner->worker_options);
-  _wait_for_startup_finished(self);
-  return !self->startup_failure;
-}
 
 void
 log_threaded_dest_driver_set_max_retries_on_error(LogDriver *s, gint max_retries)
@@ -998,6 +973,65 @@ _init_stats_key(LogThreadedDestDriver *self, StatsClusterKey *sc_key)
                                 self->format_stats_instance(self));
 }
 
+void
+log_threaded_dest_driver_insert_msg_length_stats(LogThreadedDestDriver *self, gsize len)
+{
+  stats_aggregator_insert_data(self->max_message_size, len);
+  stats_aggregator_insert_data(self->average_messages_size, len);
+}
+
+void
+log_threaded_dest_driver_insert_batch_length_stats(LogThreadedDestDriver *self, gsize len)
+{
+  stats_aggregator_insert_data(self->max_batch_size, len);
+  stats_aggregator_insert_data(self->average_batch_size, len);
+}
+
+void
+log_threaded_dest_driver_register_aggregated_stats(LogThreadedDestDriver *self)
+{
+  StatsClusterKey sc_key_eps_input;
+  _init_stats_key(self, &sc_key_eps_input);
+  stats_aggregator_lock();
+  StatsClusterKey sc_key;
+
+  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                         self->format_stats_instance(self), "msg_size_max");
+  stats_register_aggregator_maximum(0, &sc_key, &self->max_message_size);
+
+  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                         self->format_stats_instance(self), "msg_size_avg");
+  stats_register_aggregator_average(0, &sc_key, &self->average_messages_size);
+
+  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                         self->format_stats_instance(self), "batch_size_max");
+  stats_register_aggregator_maximum(0, &sc_key, &self->max_batch_size);
+
+  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                         self->format_stats_instance(self), "batch_size_avg");
+  stats_register_aggregator_average(0, &sc_key, &self->average_batch_size);
+
+  stats_cluster_single_key_set_with_name(&sc_key, self->stats_source | SCS_DESTINATION, self->super.super.id,
+                                         self->format_stats_instance(self), "eps");
+  stats_register_aggregator_cps(0, &sc_key, &sc_key_eps_input, SC_TYPE_WRITTEN, &self->CPS);
+
+  stats_aggregator_unlock();
+}
+
+void
+log_threaded_dest_driver_unregister_aggregated_stats(LogThreadedDestDriver *self)
+{
+  stats_aggregator_lock();
+
+  stats_unregister_aggregator_maximum(&self->max_message_size);
+  stats_unregister_aggregator_average(&self->average_messages_size);
+  stats_unregister_aggregator_maximum(&self->max_batch_size);
+  stats_unregister_aggregator_average(&self->average_batch_size);
+  stats_unregister_aggregator_cps(&self->CPS);
+
+  stats_aggregator_unlock();
+}
+
 static void
 _register_stats(LogThreadedDestDriver *self)
 {
@@ -1009,6 +1043,7 @@ _register_stats(LogThreadedDestDriver *self)
     stats_register_counter(0, &sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
     stats_register_counter(0, &sc_key, SC_TYPE_PROCESSED, &self->processed_messages);
     stats_register_counter(0, &sc_key, SC_TYPE_WRITTEN, &self->written_messages);
+
   }
   stats_unlock();
 }
@@ -1024,6 +1059,7 @@ _unregister_stats(LogThreadedDestDriver *self)
     stats_unregister_counter(&sc_key, SC_TYPE_DROPPED, &self->dropped_messages);
     stats_unregister_counter(&sc_key, SC_TYPE_PROCESSED, &self->processed_messages);
     stats_unregister_counter(&sc_key, SC_TYPE_WRITTEN, &self->written_messages);
+
   }
   stats_unlock();
 }
@@ -1072,8 +1108,9 @@ log_threaded_dest_driver_init_method(LogPipe *s)
   if (self->time_reopen == -1)
     self->time_reopen = cfg->time_reopen;
 
-  self->shared_seq_num = GPOINTER_TO_INT(cfg_persist_config_fetch(cfg,
-                                         _format_seqnum_persist_name(self)));
+  gpointer persisted_value = cfg_persist_config_fetch(cfg, _format_seqnum_persist_name(self));
+  self->shared_seq_num = GPOINTER_TO_INT(persisted_value);
+
   if (!self->shared_seq_num)
     init_sequence_number(&self->shared_seq_num);
 
@@ -1096,7 +1133,7 @@ log_threaded_dest_driver_start_workers(LogPipe *s)
 
   for (gint worker_index = 0; worker_index < self->num_workers; worker_index++)
     {
-      if (!_start_worker_thread(self->workers[worker_index]))
+      if (!log_threaded_dest_worker_start(self->workers[worker_index]))
         return FALSE;
     }
   return TRUE;
@@ -1132,7 +1169,7 @@ log_threaded_dest_driver_free(LogPipe *s)
   LogThreadedDestDriver *self = (LogThreadedDestDriver *)s;
 
   log_threaded_dest_worker_free_method(&self->worker.instance);
-  g_mutex_free(self->lock);
+  g_mutex_clear(&self->lock);
   g_free(self->workers);
   log_dest_driver_free((LogPipe *)self);
 }
@@ -1142,8 +1179,6 @@ log_threaded_dest_driver_init_instance(LogThreadedDestDriver *self, GlobalConfig
 {
   log_dest_driver_init_instance(&self->super, cfg);
 
-  self->worker_options.is_output_thread = TRUE;
-
   self->super.super.super.init = log_threaded_dest_driver_init_method;
   self->super.super.super.deinit = log_threaded_dest_driver_deinit_method;
   self->super.super.super.queue = log_threaded_dest_driver_queue;
@@ -1157,7 +1192,7 @@ log_threaded_dest_driver_init_instance(LogThreadedDestDriver *self, GlobalConfig
 
   self->retries_on_error_max = MAX_RETRIES_ON_ERROR_DEFAULT;
   self->retries_max = MAX_RETRIES_BEFORE_SUSPEND_DEFAULT;
-  self->lock = g_mutex_new();
+  g_mutex_init(&self->lock);
   log_threaded_dest_worker_init_instance(&self->worker.instance, self, 0);
   _init_worker_compat_layer(&self->worker.instance);
 }
