@@ -23,6 +23,7 @@
  */
 
 #include <criterion/criterion.h>
+#include "libtest/cr_template.h"
 
 #include "logthrsource/logthrfetcherdrv.h"
 #include "apphook.h"
@@ -31,7 +32,6 @@
 #include "cfg.h"
 #include "stats/stats-counter.h"
 #include "logsource.h"
-#include "cr_template.h"
 
 typedef struct _TestThreadedFetcherDriver
 {
@@ -131,7 +131,6 @@ start_test_threaded_fetcher(TestThreadedFetcherDriver *s)
 {
   cr_assert(log_pipe_init(&s->super.super.super.super.super));
   cr_assert(log_pipe_on_config_inited(&s->super.super.super.super.super));
-  app_config_changed();
 }
 
 static void
