@@ -48,7 +48,7 @@ _engage(MainLoopIOWorkerJob *self)
 
 /* NOTE: runs in the main thread */
 void
-main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self)
+main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self, GIOCondition cond)
 {
   g_assert(self->working == FALSE);
   if (main_loop_workers_quit)
@@ -57,6 +57,7 @@ main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self)
   _engage(self);
   main_loop_worker_job_start();
   self->working = TRUE;
+  self->cond = cond;
   iv_work_pool_submit_work(&main_loop_io_workers, &self->work_item);
 }
 
@@ -65,7 +66,7 @@ main_loop_io_worker_job_submit(MainLoopIOWorkerJob *self)
 static void
 _work(MainLoopIOWorkerJob *self)
 {
-  self->work(self->user_data);
+  self->work(self->user_data, self->cond);
   main_loop_worker_invoke_batch_callbacks();
   main_loop_worker_run_gc();
 }
@@ -99,6 +100,18 @@ get_processor_count(void)
 #endif
 }
 
+static void
+main_loop_io_worker_thread_start(void *cookie)
+{
+  main_loop_worker_thread_start(MLW_ASYNC_WORKER);
+}
+
+static void
+main_loop_io_worker_thread_stop(void *cookie)
+{
+  main_loop_worker_thread_stop();
+}
+
 void
 main_loop_io_worker_init(void)
 {
@@ -108,8 +121,8 @@ main_loop_io_worker_init(void)
                                              MAIN_LOOP_MAX_WORKER_THREADS);
     }
 
-  main_loop_io_workers.thread_start = (void (*)(void *)) main_loop_worker_thread_start;
-  main_loop_io_workers.thread_stop = (void (*)(void *)) main_loop_worker_thread_stop;
+  main_loop_io_workers.thread_start = main_loop_io_worker_thread_start;
+  main_loop_io_workers.thread_stop = main_loop_io_worker_thread_stop;
   iv_work_pool_create(&main_loop_io_workers);
 
   log_queue_set_max_threads(MIN(main_loop_io_workers.max_threads, MAIN_LOOP_MAX_WORKER_THREADS));
