@@ -91,7 +91,7 @@ msg_format_postprocess_message(MsgFormatOptions *options, LogMessage *msg,
       gchar *p;
 
       p = msg_text = (gchar *) log_msg_get_value(msg, LM_V_MESSAGE, &msg_len);
-      while ((p = find_cr_or_lf(p, msg_text + msg_len - p)))
+      while ((p = find_cr_or_lf_or_nul(p, msg_text + msg_len - p)))
         {
           *p = ' ';
           p++;
@@ -121,9 +121,9 @@ msg_format_process_message(MsgFormatOptions *options, LogMessage *msg,
 }
 
 gboolean
-msg_format_parse_conditional(MsgFormatOptions *options, LogMessage *msg,
-                             const guchar *data, gsize length,
-                             gsize *problem_position)
+msg_format_try_parse_into(MsgFormatOptions *options, LogMessage *msg,
+                          const guchar *data, gsize length,
+                          gsize *problem_position)
 {
   if (G_UNLIKELY(!options->format_handler))
     {
@@ -144,12 +144,12 @@ msg_format_parse_conditional(MsgFormatOptions *options, LogMessage *msg,
 }
 
 void
-msg_format_parse(MsgFormatOptions *options, LogMessage *msg,
-                 const guchar *data, gsize length)
+msg_format_parse_into(MsgFormatOptions *options, LogMessage *msg,
+                      const guchar *data, gsize length)
 {
   gsize problem_position = 0;
 
-  if (!msg_format_parse_conditional(options, msg, data, length, &problem_position))
+  if (!msg_format_try_parse_into(options, msg, data, length, &problem_position))
     {
       msg_format_inject_parse_error(msg, data, _rstripped_message_length(data, length), problem_position);
 
@@ -158,6 +158,36 @@ msg_format_parse(MsgFormatOptions *options, LogMessage *msg,
     }
 }
 
+static gsize
+_determine_payload_size(MsgFormatOptions *parse_options, const guchar *data, gsize length)
+{
+  gsize payload_size;
+
+  if ((parse_options->flags & LP_STORE_RAW_MESSAGE))
+    payload_size = length * 4;
+  else
+    payload_size = length * 2;
+
+  return MAX(payload_size, 256);
+}
+
+LogMessage *
+msg_format_construct_message(MsgFormatOptions *options, const guchar *data, gsize length)
+{
+  LogMessage *msg = log_msg_sized_new(_determine_payload_size(options, data, length));
+  return msg;
+}
+
+LogMessage *
+msg_format_parse(MsgFormatOptions *options, const guchar *data, gsize length)
+{
+  LogMessage *msg = msg_format_construct_message(options, data, length);
+
+  msg_trace("Initial message parsing follows");
+  msg_format_parse_into(options, msg, data, length);
+  return msg;
+}
+
 void
 msg_format_options_defaults(MsgFormatOptions *options)
 {
@@ -243,7 +273,7 @@ CfgFlagHandler msg_format_flag_handlers[] =
   { "no-hostname",              CFH_CLEAR, offsetof(MsgFormatOptions, flags), LP_EXPECT_HOSTNAME },
   { "guess-timezone",             CFH_SET, offsetof(MsgFormatOptions, flags), LP_GUESS_TIMEZONE },
   { "no-header",                  CFH_SET, offsetof(MsgFormatOptions, flags), LP_NO_HEADER },
-
+  { "no-rfc3164-fallback",        CFH_SET, offsetof(MsgFormatOptions, flags), LP_NO_RFC3164_FALLBACK },
   { NULL },
 };
 
