@@ -70,7 +70,7 @@ extern int parser_expr_debug;
     PARSER_EXPR_EMPTY = -2,
     PARSER_EXPR_EOF = 0,           /* "end of file"  */
     PARSER_EXPR_error = 256,       /* error  */
-    PARSER_EXPR_UNDEF = 10514,     /* "invalid token"  */
+    PARSER_EXPR_UNDEF = 10523,     /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -212,6 +212,7 @@ extern int parser_expr_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -222,9 +223,10 @@ extern int parser_expr_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513 /* KW_FETCH_NO_DATA_DELAY  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522 /* KW_FETCH_NO_DATA_DELAY  */
   };
   typedef enum parser_expr_tokentype parser_expr_token_kind_t;
 #endif
@@ -232,7 +234,7 @@ extern int parser_expr_debug;
 #define PARSER_EXPR_EMPTY -2
 #define PARSER_EXPR_EOF 0
 #define PARSER_EXPR_error 256
-#define PARSER_EXPR_UNDEF 10514
+#define PARSER_EXPR_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -374,6 +376,7 @@ extern int parser_expr_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -384,9 +387,10 @@ extern int parser_expr_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
 
 /* Value type.  */
 #if ! defined PARSER_EXPR_STYPE && ! defined PARSER_EXPR_STYPE_IS_DECLARED
