@@ -69,7 +69,7 @@ extern int rewrite_expr_debug;
     REWRITE_EXPR_EMPTY = -2,
     REWRITE_EXPR_EOF = 0,          /* "end of file"  */
     REWRITE_EXPR_error = 256,      /* error  */
-    REWRITE_EXPR_UNDEF = 10514,    /* "invalid token"  */
+    REWRITE_EXPR_UNDEF = 10523,    /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -211,6 +211,7 @@ extern int rewrite_expr_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -221,20 +222,24 @@ extern int rewrite_expr_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_SET_TAG = 10515,            /* KW_SET_TAG  */
-    KW_CLEAR_TAG = 10516,          /* KW_CLEAR_TAG  */
-    KW_GROUP_SET = 10517,          /* KW_GROUP_SET  */
-    KW_GROUP_UNSET = 10518,        /* KW_GROUP_UNSET  */
-    KW_SET = 10519,                /* KW_SET  */
-    KW_UNSET = 10520,              /* KW_UNSET  */
-    KW_SUBST = 10521,              /* KW_SUBST  */
-    KW_VALUES = 10522,             /* KW_VALUES  */
-    KW_SET_SEVERITY = 10523,       /* KW_SET_SEVERITY  */
-    KW_SET_FACILITY = 10524,       /* KW_SET_FACILITY  */
-    KW_SET_PRI = 10525             /* KW_SET_PRI  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_SET_TAG = 10524,            /* KW_SET_TAG  */
+    KW_CLEAR_TAG = 10525,          /* KW_CLEAR_TAG  */
+    KW_GROUP_SET = 10526,          /* KW_GROUP_SET  */
+    KW_GROUP_UNSET = 10527,        /* KW_GROUP_UNSET  */
+    KW_SET = 10528,                /* KW_SET  */
+    KW_UNSET = 10529,              /* KW_UNSET  */
+    KW_RENAME = 10530,             /* KW_RENAME  */
+    KW_SUBST = 10531,              /* KW_SUBST  */
+    KW_VALUES = 10532,             /* KW_VALUES  */
+    KW_SET_SEVERITY = 10533,       /* KW_SET_SEVERITY  */
+    KW_SET_FACILITY = 10534,       /* KW_SET_FACILITY  */
+    KW_SET_PRI = 10535,            /* KW_SET_PRI  */
+    KW_SET_MATCHES = 10536,        /* KW_SET_MATCHES  */
+    KW_UNSET_MATCHES = 10537       /* KW_UNSET_MATCHES  */
   };
   typedef enum rewrite_expr_tokentype rewrite_expr_token_kind_t;
 #endif
@@ -242,7 +247,7 @@ extern int rewrite_expr_debug;
 #define REWRITE_EXPR_EMPTY -2
 #define REWRITE_EXPR_EOF 0
 #define REWRITE_EXPR_error 256
-#define REWRITE_EXPR_UNDEF 10514
+#define REWRITE_EXPR_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -384,6 +389,7 @@ extern int rewrite_expr_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -394,20 +400,24 @@ extern int rewrite_expr_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_SET_TAG 10515
-#define KW_CLEAR_TAG 10516
-#define KW_GROUP_SET 10517
-#define KW_GROUP_UNSET 10518
-#define KW_SET 10519
-#define KW_UNSET 10520
-#define KW_SUBST 10521
-#define KW_VALUES 10522
-#define KW_SET_SEVERITY 10523
-#define KW_SET_FACILITY 10524
-#define KW_SET_PRI 10525
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_SET_TAG 10524
+#define KW_CLEAR_TAG 10525
+#define KW_GROUP_SET 10526
+#define KW_GROUP_UNSET 10527
+#define KW_SET 10528
+#define KW_UNSET 10529
+#define KW_RENAME 10530
+#define KW_SUBST 10531
+#define KW_VALUES 10532
+#define KW_SET_SEVERITY 10533
+#define KW_SET_FACILITY 10534
+#define KW_SET_PRI 10535
+#define KW_SET_MATCHES 10536
+#define KW_UNSET_MATCHES 10537
 
 /* Value type.  */
 #if ! defined REWRITE_EXPR_STYPE && ! defined REWRITE_EXPR_STYPE_IS_DECLARED
