@@ -25,6 +25,7 @@
 
 #include "rewrite-groupset.h"
 #include "scratch-buffers.h"
+#include "string-list.h"
 
 typedef struct _LogRewriteGroupSetCallbackData
 {
@@ -34,7 +35,7 @@ typedef struct _LogRewriteGroupSetCallbackData
 
 /* TODO escape '\0' when passing down the value */
 static gboolean
-log_rewrite_groupset_foreach_func(const gchar *name, TypeHint type,
+log_rewrite_groupset_foreach_func(const gchar *name, LogMessageValueType type,
                                   const gchar *value, gsize value_len,
                                   gpointer user_data)
 {
@@ -42,28 +43,28 @@ log_rewrite_groupset_foreach_func(const gchar *name, TypeHint type,
   LogMessage *msg = callback_data->msg;
   LogTemplate *template = callback_data->template;
   GString *result;
+  ScratchBuffersMarker mark;
 
-  result = scratch_buffers_alloc();
+  result = scratch_buffers_alloc_and_mark(&mark);
 
-  LogTemplateEvalOptions options = {NULL, LTZ_LOCAL, 0, value};
-  log_template_format(template, msg, &options, result);
+  LogTemplateEvalOptions options = {NULL, LTZ_LOCAL, 0, value, type};
+  log_template_format_value_and_type(template, msg, &options, result, &type);
 
-  NVHandle handle = log_msg_get_value_handle(name);
-  log_msg_set_value(msg, handle, result->str, result->len);
+  log_msg_set_value_by_name_with_type(msg, name, result->str, result->len, type);
 
+  scratch_buffers_reclaim_marked(mark);
   return FALSE;
 }
 
 static gboolean
-log_rewrite_groupunset_foreach_func(const gchar *name, TypeHint type,
+log_rewrite_groupunset_foreach_func(const gchar *name, LogMessageValueType type,
                                     const gchar *value, gsize value_len,
                                     gpointer user_data)
 {
   LogRewriteGroupSetCallbackData *callback_data = (LogRewriteGroupSetCallbackData *) user_data;
   LogMessage *msg = callback_data->msg;
 
-  NVHandle handle = log_msg_get_value_handle(name);
-  log_msg_unset_value(msg, handle);
+  log_msg_unset_value_by_name(msg, name);
   return FALSE;
 }
 
@@ -77,29 +78,22 @@ log_rewrite_groupset_process(LogRewrite *s, LogMessage **msg, const LogPathOptio
   log_msg_make_writable(msg, path_options);
   userdata.msg = *msg;
   userdata.template = self->replacement;
-  LogTemplateEvalOptions options = {&cfg->template_options, LTZ_LOCAL, 0, NULL};
+  LogTemplateEvalOptions options = {&cfg->template_options, LTZ_LOCAL, 0, NULL, LM_VT_STRING};
   value_pairs_foreach(self->query, self->vp_func, *msg, &options, &userdata);
 }
 
-static void
-__free_field(gpointer field, gpointer user_data)
-{
-  g_free(field);
-}
-
-
 void
 log_rewrite_groupset_add_fields(LogRewrite *rewrite, GList *fields)
 {
   LogRewriteGroupSet *self = (LogRewriteGroupSet *) rewrite;
-  GList *head;
-  for (head = fields; head; head = head->next)
+
+  for (GList *head = fields; head; head = head->next)
     {
       value_pairs_add_glob_pattern(self->query, head->data, TRUE);
     }
-  g_list_foreach(fields, __free_field, NULL);
-  g_list_free(fields);
+  string_list_free(fields);
 }
+
 static LogPipe *
 log_rewrite_groupset_clone(LogPipe *s)
 {
@@ -109,12 +103,10 @@ log_rewrite_groupset_clone(LogPipe *s)
   value_pairs_unref(cloned->query);
   cloned->query = value_pairs_ref(self->query);
   cloned->vp_func = self->vp_func;
-
-  if (self->super.condition)
-    cloned->super.condition = filter_expr_clone(self->super.condition);
+  cloned->super.condition = filter_expr_clone(self->super.condition);
 
   return &cloned->super.super;
-};
+}
 
 void
 log_rewrite_groupset_free(LogPipe *s)
@@ -137,7 +129,7 @@ log_rewrite_groupset_new(LogTemplate *template, GlobalConfig *cfg)
   self->super.super.clone = log_rewrite_groupset_clone;
 
   self->replacement = log_template_ref(template);
-  self->query = value_pairs_new();
+  self->query = value_pairs_new(cfg);
   self->vp_func = log_rewrite_groupset_foreach_func;
 
   return &self->super;
@@ -147,6 +139,7 @@ LogRewrite *
 log_rewrite_groupunset_new(GlobalConfig *cfg)
 {
   LogRewriteGroupSet *self = (LogRewriteGroupSet *)log_rewrite_groupset_new(NULL, cfg);
+
   self->vp_func = log_rewrite_groupunset_foreach_func;
   return &self->super;
 }
