@@ -50,16 +50,16 @@ nondumpable_setlogger(NonDumpableLogger _debug, NonDumpableLogger _fatal)
 
 #define logger_debug(summary, fmt, ...) \
 { \
- char reason[32] = { 0 }; \
- snprintf(reason, sizeof(reason), fmt, __VA_ARGS__); \
- logger_debug_fn(summary, reason);   \
+ gchar *reason = g_strdup_printf(fmt, __VA_ARGS__); \
+ logger_debug_fn(summary, reason); \
+ g_free(reason); \
 }
 
 #define logger_fatal(summary, fmt, ...) \
 { \
- char reason[32] = { 0 }; \
- snprintf(reason, sizeof(reason), fmt, __VA_ARGS__); \
- logger_fatal_fn(summary, reason);   \
+ gchar *reason = g_strdup_printf(fmt, __VA_ARGS__); \
+ logger_fatal_fn(summary, reason); \
+ g_free(reason); \
 }
 
 typedef struct
