@@ -81,12 +81,13 @@ tf_simple_func_eval(LogTemplateFunction *self, gpointer s, LogTemplateInvokeArgs
 }
 
 void
-tf_simple_func_call(LogTemplateFunction *self, gpointer s, const LogTemplateInvokeArgs *args, GString *result)
+tf_simple_func_call(LogTemplateFunction *self, gpointer s, const LogTemplateInvokeArgs *args, GString *result,
+                    LogMessageValueType *type)
 {
   TFSimpleFunc simple_func = (TFSimpleFunc) self->arg;
   TFSimpleFuncState *state = (TFSimpleFuncState *) s;
 
-  simple_func(args->messages[args->num_messages-1], state->argc, (GString **) args->argv, result);
+  simple_func(args->messages[args->num_messages-1], state->argc, (GString **) args->argv, result, type);
 }
 
 void
