@@ -245,7 +245,6 @@ static int first_wday_of(int yr);
 #define HAVE_HOUR(s)    (s & S_HOUR)
 #define HAVE_USEC(s)    (s & S_USEC)
 
-static char gmt[] = { "GMT" };
 static char utc[] = { "UTC" };
 /* RFC-822/RFC-2822 */
 static const char *const nast[5] =
@@ -273,10 +272,11 @@ gchar *
 wall_clock_time_strptime(WallClockTime *wct, const gchar *format, const gchar *input)
 {
   unsigned char c;
-  const unsigned char *bp, *ep;
+  const unsigned char *bp, *ep, *zname;
   int alt_format, i, split_year = 0, neg = 0, state = 0,
-                     day_offset = -1, week_offset = 0, offs;
+                     day_offset = -1, week_offset = 0, offs, mandatory;
   const char *new_fmt;
+  const char *const *system_tznames;
 
   bp = (const unsigned char *)input;
 
@@ -607,30 +607,8 @@ recurse:
           continue;
 
         case 'Z':
-          if (strncmp((const char *)bp, gmt, 3) == 0 ||
-              strncmp((const char *)bp, utc, 3) == 0)
-            {
-              wct->tm.tm_isdst = 0;
-              wct->wct_gmtoff = 0;
-              wct->wct_zone = gmt;
-              bp += 3;
-            }
-          else
-            {
-              ep = find_string(bp, &i, (const char *const *)tzname, NULL, 2);
-              if (ep != NULL)
-                {
-                  wct->tm.tm_isdst = i;
-#ifdef SYSLOG_NG_HAVE_TIMEZONE
-                  wct->wct_gmtoff = -(timezone);
-#endif
-                  wct->wct_zone = tzname[i];
-                }
-              bp = ep;
-            }
-          continue;
-
         case 'z':
+          mandatory = c == 'z';
           /*
            * We recognize all ISO 8601 formats:
            * Z  = Zulu time/UTC
@@ -648,9 +626,11 @@ recurse:
            * [A-IL-M] = -1 ... -9 (J not used)
            * [N-Y]  = +1 ... +12
            */
-          while (isspace(*bp))
-            bp++;
+          if (mandatory)
+            while (isspace(*bp))
+              bp++;
 
+          zname = bp;
           switch (*bp++)
             {
             case 'G':
@@ -691,6 +671,17 @@ recurse:
                   bp = ep;
                   continue;
                 }
+              system_tznames = cached_get_system_tznames();
+              ep = find_string(bp, &i, system_tznames, NULL, 2);
+              if (ep != NULL)
+                {
+                  wct->tm.tm_isdst = i;
+                  wct->wct_gmtoff = -cached_get_system_tzofs() + wct->tm.tm_isdst*3600;
+                  wct->wct_zone = system_tznames[i];
+                  bp = ep;
+                  continue;
+                }
+
 
               if ((*bp >= 'A' && *bp <= 'I') ||
                   (*bp >= 'L' && *bp <= 'Y'))
@@ -707,7 +698,11 @@ recurse:
                   bp++;
                   continue;
                 }
-              return NULL;
+              if (mandatory)
+                return NULL;
+
+              bp = zname;
+              continue;
             }
           offs = 0;
           for (i = 0; i < 4; )
@@ -728,22 +723,27 @@ recurse:
           switch (i)
             {
             case 2:
-              offs *= 100;
+              offs *= 3600;
               break;
             case 4:
               i = offs % 100;
+              offs /= 100;
               if (i >= 60)
-                return NULL;
+                goto out;
               /* Convert minutes into decimal */
-              offs = (offs / 100) * 100 + (i * 50) / 30;
+              offs = offs * 3600 + i * 60;
               break;
             default:
-              return NULL;
+out:
+              if (mandatory)
+                return NULL;
+              bp = zname;
+              continue;
             }
           if (neg)
             offs = -offs;
           wct->tm.tm_isdst = 0; /* XXX */
-          wct->wct_gmtoff = (offs * 3600) / 100;
+          wct->wct_gmtoff = offs;
           wct->wct_zone = utc; /* XXX */
           continue;
 
@@ -873,6 +873,53 @@ wall_clock_time_guess_missing_year(WallClockTime *self)
     }
 }
 
+void
+wall_clock_time_guess_missing_fields(WallClockTime *self)
+{
+  /*
+   * The missing cases for date can be divided into three types:
+   * 1) missing all fileds -> use current date
+   * 2) only miss year -> guess year based on current year and month (current
+   * year, last year or next year)
+   * 3) the rest of the cases don't make much sense, so zero initialization of
+   * the missing field makes sense. And the year is initializeed to the current
+   *  one.
+   */
+  time_t now;
+  struct tm tm;
+
+  now = cached_g_current_time_sec();
+  cached_localtime(&now, &tm);
+
+  if (self->wct_year == -1 && self->wct_mon == -1 && self->wct_mday == -1)
+    {
+      self->wct_year = tm.tm_year;
+      self->wct_mon = tm.tm_mon;
+      self->wct_mday = tm.tm_mday;
+    }
+  else if (self->wct_year == -1 && self->wct_mon != -1 && self->wct_mday != -1)
+    {
+      self->wct_year = determine_year_for_month(self->wct_mon, &tm);
+    }
+  else
+    {
+      if (self->wct_year == -1)
+        self->wct_year = tm.tm_year;
+      if (self->wct_mon == -1)
+        self->wct_mon = 0;
+      if (self->wct_mday == -1)
+        self->wct_mday = 1;  // day of the month - [1, 31]
+    }
+
+  if (self->wct_hour == -1)
+    self->wct_hour = 0;
+  if (self->wct_min == -1)
+    self->wct_min = 0;
+  if (self->wct_sec == -1)
+    self->wct_sec = 0;
+}
+
+
 /*
  * Calculate the week day of the first day of a year. Valid for
  * the Gregorian calendar, which began Sept 14, 1752 in the UK
