@@ -61,7 +61,7 @@ extern int afprog_debug;
     AFPROG_EMPTY = -2,
     AFPROG_EOF = 0,                /* "end of file"  */
     AFPROG_error = 256,            /* error  */
-    AFPROG_UNDEF = 10514,          /* "invalid token"  */
+    AFPROG_UNDEF = 10523,          /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int afprog_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,12 +214,13 @@ extern int afprog_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_PROGRAM = 10515,            /* KW_PROGRAM  */
-    KW_KEEP_ALIVE = 10516,         /* KW_KEEP_ALIVE  */
-    KW_INHERIT_ENVIRONMENT = 10517 /* KW_INHERIT_ENVIRONMENT  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_PROGRAM = 10524,            /* KW_PROGRAM  */
+    KW_KEEP_ALIVE = 10525,         /* KW_KEEP_ALIVE  */
+    KW_INHERIT_ENVIRONMENT = 10526 /* KW_INHERIT_ENVIRONMENT  */
   };
   typedef enum afprog_tokentype afprog_token_kind_t;
 #endif
@@ -226,7 +228,7 @@ extern int afprog_debug;
 #define AFPROG_EMPTY -2
 #define AFPROG_EOF 0
 #define AFPROG_error 256
-#define AFPROG_UNDEF 10514
+#define AFPROG_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -368,6 +370,7 @@ extern int afprog_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -378,12 +381,13 @@ extern int afprog_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_PROGRAM 10515
-#define KW_KEEP_ALIVE 10516
-#define KW_INHERIT_ENVIRONMENT 10517
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_PROGRAM 10524
+#define KW_KEEP_ALIVE 10525
+#define KW_INHERIT_ENVIRONMENT 10526
 
 /* Value type.  */
 #if ! defined AFPROG_STYPE && ! defined AFPROG_STYPE_IS_DECLARED
