@@ -257,8 +257,10 @@ static void
 _smtp_message_add_recipient_from_template(smtp_message_t self, AFSMTPDriver *driver, LogTemplate *template,
                                           LogMessage *msg)
 {
-  log_template_format(template, msg, &driver->template_options, LTZ_SEND,
-                      driver->super.worker.instance.seq_num, NULL, driver->str);
+  LogTemplateEvalOptions options = {&driver->template_options, LTZ_SEND,
+                                    driver->super.worker.instance.seq_num, NULL, LM_VT_STRING
+                                   };
+  log_template_format(template, msg, &options, driver->str);
   smtp_add_recipient(self, afsmtp_wash_string (driver->str->str));
 }
 
@@ -280,8 +282,10 @@ afsmtp_dd_msg_add_header(AFSMTPHeader *hdr, gpointer user_data)
   LogMessage *msg = ((gpointer *)user_data)[1];
   smtp_message_t message = ((gpointer *)user_data)[2];
 
-  log_template_format(hdr->template, msg, &self->template_options, LTZ_LOCAL,
-                      self->super.worker.instance.seq_num, NULL, self->str);
+  LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL,
+                                    self->super.worker.instance.seq_num, NULL, LM_VT_STRING
+                                   };
+  log_template_format(hdr->template, msg, &options, self->str);
 
   smtp_set_header(message, hdr->name, afsmtp_wash_string (self->str->str), NULL);
   smtp_set_header_option(message, hdr->name, Hdr_OVERRIDE, 1);
@@ -354,17 +358,17 @@ afsmtp_dd_cb_monitor(const gchar *buf, gint buflen, gint writing,
     case SMTP_CB_READING:
       msg_debug ("SMTP Session: SERVER",
                  evt_tag_str("driver", self->super.super.super.id),
-                 evt_tag_printf("message", "%.*s", buflen, buf));
+                 evt_tag_mem("message", buf, buflen));
       break;
     case SMTP_CB_WRITING:
       msg_debug("SMTP Session: CLIENT",
                 evt_tag_str("driver", self->super.super.super.id),
-                evt_tag_printf("message", "%.*s", buflen, buf));
+                evt_tag_mem("message", buf, buflen));
       break;
     case SMTP_CB_HEADERS:
       msg_debug("SMTP Session: HEADERS",
                 evt_tag_str("driver", self->super.super.super.id),
-                evt_tag_printf("data", "%.*s", buflen, buf));
+                evt_tag_mem("data", buf, buflen));
       break;
     }
 }
@@ -393,16 +397,20 @@ __build_message(AFSMTPDriver *self, LogMessage *msg, smtp_session_t session)
 
   message = smtp_add_message(session);
 
-  log_template_format(self->mail_from->template, msg, &self->template_options, LTZ_SEND,
-                      self->super.worker.instance.seq_num, NULL, self->str);
+  LogTemplateEvalOptions options = {&self->template_options, LTZ_SEND,
+                                    self->super.worker.instance.seq_num, NULL, LM_VT_STRING
+                                   };
+  log_template_format(self->mail_from->template, msg, &options, self->str);
   smtp_set_reverse_path(message, afsmtp_wash_string(self->str->str));
 
   /* Defaults */
   smtp_set_header(message, "To", NULL, NULL);
   smtp_set_header(message, "From", NULL, NULL);
 
-  log_template_format(self->subject_template, msg, &self->template_options, LTZ_SEND,
-                      self->super.worker.instance.seq_num, NULL, self->str);
+  LogTemplateEvalOptions eval_options = {&self->template_options, LTZ_SEND,
+                                         self->super.worker.instance.seq_num, NULL, LM_VT_STRING
+                                        };
+  log_template_format(self->subject_template, msg, &eval_options, self->str);
   smtp_set_header(message, "Subject", afsmtp_wash_string(self->str->str));
   smtp_set_header_option(message, "Subject", Hdr_OVERRIDE, 1);
 
@@ -422,9 +430,7 @@ __build_message(AFSMTPDriver *self, LogMessage *msg, smtp_session_t session)
    * recognise headers, and will append them to the end of the body.
    */
   g_string_assign(self->str, "X-Mailer: syslog-ng " SYSLOG_NG_VERSION "\r\n\r\n");
-  log_template_append_format(self->body_template, msg, &self->template_options,
-                             LTZ_SEND, self->super.worker.instance.seq_num,
-                             NULL, self->str);
+  log_template_append_format(self->body_template, msg, &eval_options, self->str);
   smtp_set_message_str(message, self->str->str);
   return message;
 }
@@ -594,9 +600,6 @@ afsmtp_dd_init(LogPipe *s)
   AFSMTPDriver *self = (AFSMTPDriver *)s;
   GlobalConfig *cfg = log_pipe_get_config(s);
 
-  if (!log_threaded_dest_driver_init_method(s))
-    return FALSE;
-
   msg_verbose("Initializing SMTP destination",
               evt_tag_str("driver", self->super.super.super.id),
               evt_tag_str("host", self->host),
@@ -605,8 +608,11 @@ afsmtp_dd_init(LogPipe *s)
   if (!__check_required_options(self))
     return FALSE;
 
+  if (!log_threaded_dest_driver_init_method(s))
+    return FALSE;
+
   log_template_options_init(&self->template_options, cfg);
-  return log_threaded_dest_driver_start_workers(&self->super);
+  return TRUE;
 }
 
 static void
