@@ -409,7 +409,7 @@ snmpdest_worker_insert(LogThreadedDestDriver *s, LogMessage *msg)
   GList *snmp_template = self->snmp_templates;
   GString *fs = g_string_sized_new(128);
 
-  LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, 0, NULL};
+  LogTemplateEvalOptions options = {&self->template_options, LTZ_LOCAL, 0, NULL, LM_VT_STRING};
   /* go through the snmp objects and add them to the message */
   while (snmp_obj)
     {
@@ -759,7 +759,6 @@ snmpdest_dd_new(GlobalConfig *cfg)
   self->transport = g_strdup("UDP");
 
   log_template_options_defaults(&self->template_options);
-  self->worker_options.is_output_thread = TRUE;
 
   return (LogDriver *)self;
 }
