@@ -22,7 +22,6 @@
  */
 #include "transport-mapper-unix.h"
 #include "transport-unix-socket.h"
-#include "unix-credentials.h"
 #include "stats/stats-registry.h"
 
 #include <sys/types.h>
@@ -33,12 +32,10 @@
 struct _TransportMapperUnix
 {
   TransportMapper super;
-  gchar *filename;
-  gboolean pass_unix_credentials;
 };
 
 static LogTransport *
-_create_log_transport(TransportMapper *s, gint fd)
+_construct_log_transport(TransportMapper *s, gint fd)
 {
   if (s->sock_type == SOCK_DGRAM)
     return log_transport_unix_dgram_socket_new(fd);
@@ -46,18 +43,6 @@ _create_log_transport(TransportMapper *s, gint fd)
     return log_transport_unix_stream_socket_new(fd);
 }
 
-static LogTransport *
-_construct_log_transport(TransportMapper *s, gint fd)
-{
-  TransportMapperUnix *self = (TransportMapperUnix *) s;
-  LogTransport *transport = _create_log_transport(s, fd);
-
-  if (self->pass_unix_credentials)
-    socket_set_pass_credentials(fd);
-
-  return transport;
-}
-
 static TransportMapperUnix *
 transport_mapper_unix_new_instance(const gchar *transport, gint sock_type)
 {
@@ -70,13 +55,6 @@ transport_mapper_unix_new_instance(const gchar *transport, gint sock_type)
   return self;
 }
 
-void
-transport_mapper_unix_set_pass_unix_credentials(TransportMapper *s, gboolean pass)
-{
-  TransportMapperUnix *self = (TransportMapperUnix *) s;
-  self->pass_unix_credentials = pass;
-}
-
 TransportMapper *
 transport_mapper_unix_dgram_new(void)
 {
