@@ -61,7 +61,7 @@ extern int afsql_debug;
     AFSQL_EMPTY = -2,
     AFSQL_EOF = 0,                 /* "end of file"  */
     AFSQL_error = 256,             /* error  */
-    AFSQL_UNDEF = 10514,           /* "invalid token"  */
+    AFSQL_UNDEF = 10523,           /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int afsql_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,23 +214,24 @@ extern int afsql_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_DEFAULT = 10515,            /* KW_DEFAULT  */
-    KW_DBD_OPTION = 10516,         /* KW_DBD_OPTION  */
-    KW_INDEXES = 10517,            /* KW_INDEXES  */
-    KW_VALUES = 10518,             /* KW_VALUES  */
-    KW_SQL = 10519,                /* KW_SQL  */
-    KW_PASSWORD = 10520,           /* KW_PASSWORD  */
-    KW_USERNAME = 10521,           /* KW_USERNAME  */
-    KW_DATABASE = 10522,           /* KW_DATABASE  */
-    KW_TABLE = 10523,              /* KW_TABLE  */
-    KW_SESSION_STATEMENTS = 10524, /* KW_SESSION_STATEMENTS  */
-    KW_CREATE_STATEMENT_APPEND = 10525, /* KW_CREATE_STATEMENT_APPEND  */
-    KW_COLUMNS = 10526,            /* KW_COLUMNS  */
-    KW_NULL = 10527,               /* KW_NULL  */
-    KW_IGNORE_TNS_CONFIG = 10528   /* KW_IGNORE_TNS_CONFIG  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_DEFAULT = 10524,            /* KW_DEFAULT  */
+    KW_DBD_OPTION = 10525,         /* KW_DBD_OPTION  */
+    KW_INDEXES = 10526,            /* KW_INDEXES  */
+    KW_VALUES = 10527,             /* KW_VALUES  */
+    KW_SQL = 10528,                /* KW_SQL  */
+    KW_PASSWORD = 10529,           /* KW_PASSWORD  */
+    KW_USERNAME = 10530,           /* KW_USERNAME  */
+    KW_DATABASE = 10531,           /* KW_DATABASE  */
+    KW_TABLE = 10532,              /* KW_TABLE  */
+    KW_SESSION_STATEMENTS = 10533, /* KW_SESSION_STATEMENTS  */
+    KW_CREATE_STATEMENT_APPEND = 10534, /* KW_CREATE_STATEMENT_APPEND  */
+    KW_COLUMNS = 10535,            /* KW_COLUMNS  */
+    KW_NULL = 10536,               /* KW_NULL  */
+    KW_IGNORE_TNS_CONFIG = 10537   /* KW_IGNORE_TNS_CONFIG  */
   };
   typedef enum afsql_tokentype afsql_token_kind_t;
 #endif
@@ -237,7 +239,7 @@ extern int afsql_debug;
 #define AFSQL_EMPTY -2
 #define AFSQL_EOF 0
 #define AFSQL_error 256
-#define AFSQL_UNDEF 10514
+#define AFSQL_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -379,6 +381,7 @@ extern int afsql_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -389,23 +392,24 @@ extern int afsql_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_DEFAULT 10515
-#define KW_DBD_OPTION 10516
-#define KW_INDEXES 10517
-#define KW_VALUES 10518
-#define KW_SQL 10519
-#define KW_PASSWORD 10520
-#define KW_USERNAME 10521
-#define KW_DATABASE 10522
-#define KW_TABLE 10523
-#define KW_SESSION_STATEMENTS 10524
-#define KW_CREATE_STATEMENT_APPEND 10525
-#define KW_COLUMNS 10526
-#define KW_NULL 10527
-#define KW_IGNORE_TNS_CONFIG 10528
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_DEFAULT 10524
+#define KW_DBD_OPTION 10525
+#define KW_INDEXES 10526
+#define KW_VALUES 10527
+#define KW_SQL 10528
+#define KW_PASSWORD 10529
+#define KW_USERNAME 10530
+#define KW_DATABASE 10531
+#define KW_TABLE 10532
+#define KW_SESSION_STATEMENTS 10533
+#define KW_CREATE_STATEMENT_APPEND 10534
+#define KW_COLUMNS 10535
+#define KW_NULL 10536
+#define KW_IGNORE_TNS_CONFIG 10537
 
 /* Value type.  */
 #if ! defined AFSQL_STYPE && ! defined AFSQL_STYPE_IS_DECLARED
