@@ -69,7 +69,7 @@ extern int afstomp_debug;
     AFSTOMP_EMPTY = -2,
     AFSTOMP_EOF = 0,               /* "end of file"  */
     AFSTOMP_error = 256,           /* error  */
-    AFSTOMP_UNDEF = 10514,         /* "invalid token"  */
+    AFSTOMP_UNDEF = 10523,         /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -211,6 +211,7 @@ extern int afstomp_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -221,16 +222,17 @@ extern int afstomp_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_STOMP = 10515,              /* KW_STOMP  */
-    KW_STOMP_DESTINATION = 10516,  /* KW_STOMP_DESTINATION  */
-    KW_PERSISTENT = 10517,         /* KW_PERSISTENT  */
-    KW_ACK = 10518,                /* KW_ACK  */
-    KW_BODY = 10519,               /* KW_BODY  */
-    KW_PASSWORD = 10520,           /* KW_PASSWORD  */
-    KW_USERNAME = 10521            /* KW_USERNAME  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_STOMP = 10524,              /* KW_STOMP  */
+    KW_STOMP_DESTINATION = 10525,  /* KW_STOMP_DESTINATION  */
+    KW_PERSISTENT = 10526,         /* KW_PERSISTENT  */
+    KW_ACK = 10527,                /* KW_ACK  */
+    KW_BODY = 10528,               /* KW_BODY  */
+    KW_PASSWORD = 10529,           /* KW_PASSWORD  */
+    KW_USERNAME = 10530            /* KW_USERNAME  */
   };
   typedef enum afstomp_tokentype afstomp_token_kind_t;
 #endif
@@ -238,7 +240,7 @@ extern int afstomp_debug;
 #define AFSTOMP_EMPTY -2
 #define AFSTOMP_EOF 0
 #define AFSTOMP_error 256
-#define AFSTOMP_UNDEF 10514
+#define AFSTOMP_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -380,6 +382,7 @@ extern int afstomp_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -390,16 +393,17 @@ extern int afstomp_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_STOMP 10515
-#define KW_STOMP_DESTINATION 10516
-#define KW_PERSISTENT 10517
-#define KW_ACK 10518
-#define KW_BODY 10519
-#define KW_PASSWORD 10520
-#define KW_USERNAME 10521
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_STOMP 10524
+#define KW_STOMP_DESTINATION 10525
+#define KW_PERSISTENT 10526
+#define KW_ACK 10527
+#define KW_BODY 10528
+#define KW_PASSWORD 10529
+#define KW_USERNAME 10530
 
 /* Value type.  */
 #if ! defined AFSTOMP_STYPE && ! defined AFSTOMP_STYPE_IS_DECLARED
