@@ -61,7 +61,7 @@ extern int afstreams_debug;
     AFSTREAMS_EMPTY = -2,
     AFSTREAMS_EOF = 0,             /* "end of file"  */
     AFSTREAMS_error = 256,         /* error  */
-    AFSTREAMS_UNDEF = 10514,       /* "invalid token"  */
+    AFSTREAMS_UNDEF = 10523,       /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int afstreams_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,11 +214,12 @@ extern int afstreams_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_DOOR = 10515,               /* KW_DOOR  */
-    KW_SUN_STREAMS = 10516         /* KW_SUN_STREAMS  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_DOOR = 10524,               /* KW_DOOR  */
+    KW_SUN_STREAMS = 10525         /* KW_SUN_STREAMS  */
   };
   typedef enum afstreams_tokentype afstreams_token_kind_t;
 #endif
@@ -225,7 +227,7 @@ extern int afstreams_debug;
 #define AFSTREAMS_EMPTY -2
 #define AFSTREAMS_EOF 0
 #define AFSTREAMS_error 256
-#define AFSTREAMS_UNDEF 10514
+#define AFSTREAMS_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -367,6 +369,7 @@ extern int afstreams_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -377,11 +380,12 @@ extern int afstreams_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_DOOR 10515
-#define KW_SUN_STREAMS 10516
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_DOOR 10524
+#define KW_SUN_STREAMS 10525
 
 /* Value type.  */
 #if ! defined AFSTREAMS_STYPE && ! defined AFSTREAMS_STYPE_IS_DECLARED
