@@ -61,7 +61,7 @@ extern int dbparser_debug;
     DBPARSER_EMPTY = -2,
     DBPARSER_EOF = 0,              /* "end of file"  */
     DBPARSER_error = 256,          /* error  */
-    DBPARSER_UNDEF = 10514,        /* "invalid token"  */
+    DBPARSER_UNDEF = 10523,        /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int dbparser_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,23 +214,24 @@ extern int dbparser_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_DB_PARSER = 10515,          /* KW_DB_PARSER  */
-    KW_GROUPING_BY = 10516,        /* KW_GROUPING_BY  */
-    KW_INJECT_MODE = 10517,        /* KW_INJECT_MODE  */
-    KW_INHERIT_MODE = 10518,       /* KW_INHERIT_MODE  */
-    KW_TIMEOUT = 10519,            /* KW_TIMEOUT  */
-    KW_TRIGGER = 10520,            /* KW_TRIGGER  */
-    KW_WHERE = 10521,              /* KW_WHERE  */
-    KW_HAVING = 10522,             /* KW_HAVING  */
-    KW_AGGREGATE = 10523,          /* KW_AGGREGATE  */
-    KW_DROP_UNMATCHED = 10524,     /* KW_DROP_UNMATCHED  */
-    KW_FILE = 10525,               /* KW_FILE  */
-    KW_PROGRAM_TEMPLATE = 10526,   /* KW_PROGRAM_TEMPLATE  */
-    KW_MESSAGE_TEMPLATE = 10527,   /* KW_MESSAGE_TEMPLATE  */
-    KW_SORT_KEY = 10528            /* KW_SORT_KEY  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_DB_PARSER = 10524,          /* KW_DB_PARSER  */
+    KW_GROUPING_BY = 10525,        /* KW_GROUPING_BY  */
+    KW_INJECT_MODE = 10526,        /* KW_INJECT_MODE  */
+    KW_INHERIT_MODE = 10527,       /* KW_INHERIT_MODE  */
+    KW_TIMEOUT = 10528,            /* KW_TIMEOUT  */
+    KW_TRIGGER = 10529,            /* KW_TRIGGER  */
+    KW_WHERE = 10530,              /* KW_WHERE  */
+    KW_HAVING = 10531,             /* KW_HAVING  */
+    KW_AGGREGATE = 10532,          /* KW_AGGREGATE  */
+    KW_DROP_UNMATCHED = 10533,     /* KW_DROP_UNMATCHED  */
+    KW_FILE = 10534,               /* KW_FILE  */
+    KW_PROGRAM_TEMPLATE = 10535,   /* KW_PROGRAM_TEMPLATE  */
+    KW_MESSAGE_TEMPLATE = 10536,   /* KW_MESSAGE_TEMPLATE  */
+    KW_SORT_KEY = 10537            /* KW_SORT_KEY  */
   };
   typedef enum dbparser_tokentype dbparser_token_kind_t;
 #endif
@@ -237,7 +239,7 @@ extern int dbparser_debug;
 #define DBPARSER_EMPTY -2
 #define DBPARSER_EOF 0
 #define DBPARSER_error 256
-#define DBPARSER_UNDEF 10514
+#define DBPARSER_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -379,6 +381,7 @@ extern int dbparser_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -389,23 +392,24 @@ extern int dbparser_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_DB_PARSER 10515
-#define KW_GROUPING_BY 10516
-#define KW_INJECT_MODE 10517
-#define KW_INHERIT_MODE 10518
-#define KW_TIMEOUT 10519
-#define KW_TRIGGER 10520
-#define KW_WHERE 10521
-#define KW_HAVING 10522
-#define KW_AGGREGATE 10523
-#define KW_DROP_UNMATCHED 10524
-#define KW_FILE 10525
-#define KW_PROGRAM_TEMPLATE 10526
-#define KW_MESSAGE_TEMPLATE 10527
-#define KW_SORT_KEY 10528
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_DB_PARSER 10524
+#define KW_GROUPING_BY 10525
+#define KW_INJECT_MODE 10526
+#define KW_INHERIT_MODE 10527
+#define KW_TIMEOUT 10528
+#define KW_TRIGGER 10529
+#define KW_WHERE 10530
+#define KW_HAVING 10531
+#define KW_AGGREGATE 10532
+#define KW_DROP_UNMATCHED 10533
+#define KW_FILE 10534
+#define KW_PROGRAM_TEMPLATE 10535
+#define KW_MESSAGE_TEMPLATE 10536
+#define KW_SORT_KEY 10537
 
 /* Value type.  */
 #if ! defined DBPARSER_STYPE && ! defined DBPARSER_STYPE_IS_DECLARED
