@@ -35,10 +35,11 @@ static CfgLexerKeyword dbparser_keywords[] =
   { "grouping_by",        KW_GROUPING_BY },
   { "file",               KW_FILE },
 
-  /* correllate options */
+  /* correlate options */
   { "inject_mode",        KW_INJECT_MODE },
   { "drop_unmatched",     KW_DROP_UNMATCHED },
   { "key",                KW_KEY },
+  { "sort_key",           KW_SORT_KEY },
   { "scope",              KW_SCOPE },
   { "timeout",            KW_TIMEOUT },
   { "aggregate",          KW_AGGREGATE },
@@ -47,6 +48,8 @@ static CfgLexerKeyword dbparser_keywords[] =
   { "having",             KW_HAVING },
   { "trigger",            KW_TRIGGER },
   { "value",              KW_VALUE },
+  { "program_template",   KW_PROGRAM_TEMPLATE },
+  { "message_template",   KW_MESSAGE_TEMPLATE },
   { NULL }
 };
 
@@ -61,4 +64,4 @@ CfgParser dbparser_parser =
   .cleanup = (void (*)(gpointer)) log_pipe_unref,
 };
 
-CFG_PARSER_IMPLEMENT_LEXER_BINDING(dbparser_, LogParser **)
+CFG_PARSER_IMPLEMENT_LEXER_BINDING(dbparser_, DBPARSER_, LogParser **)
