@@ -21,19 +21,20 @@
  *
  */
 
+#include <criterion/criterion.h>
+#include <criterion/parameterized.h>
+
 #include "patternize.h"
 #include "logmsg/logmsg.h"
 #include "cfg.h"
 #include "plugin.h"
 #include "apphook.h"
+#include "msg-format.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
-#include <criterion/criterion.h>
-#include <criterion/parameterized.h>
-
 MsgFormatOptions parse_options;
 
 static void setup(void)
@@ -65,7 +66,6 @@ _get_logmessages(const gchar *logs)
   loglinesType *self;
   gchar **input_lines;
   gchar *logline;
-  GSockAddr *addr = g_sockaddr_inet_new("10.10.10.10", 1010);
   LogMessage *msg;
 
   self = g_new(loglinesType, 1);
@@ -81,7 +81,7 @@ _get_logmessages(const gchar *logs)
       if (logline[len-1] == '\n')
         logline[len-1] = 0;
 
-      msg = log_msg_new(logline, len, addr, &parse_options);
+      msg = msg_format_parse(&parse_options, (const guchar *) logline, len);
       g_ptr_array_add(self->logmessages, msg);
       ++(self->num_of_logs);
       g_free(logline);
