@@ -28,7 +28,7 @@
 
 typedef struct _TWEntry TWEntry;
 typedef struct _TimerWheel TimerWheel;
-typedef void (*TWCallbackFunc)(TimerWheel *tw, guint64 now, gpointer user_data);
+typedef void (*TWCallbackFunc)(TimerWheel *tw, guint64 now, gpointer user_data, gpointer caller_context);
 
 TWEntry *timer_wheel_add_timer(TimerWheel *self, gint timeout, TWCallbackFunc cb, gpointer user_data,
                                GDestroyNotify user_data_free);
@@ -36,9 +36,9 @@ void timer_wheel_del_timer(TimerWheel *self, TWEntry *entry);
 void timer_wheel_mod_timer(TimerWheel *self, TWEntry *entry, gint new_timeout);
 guint64 timer_wheel_get_timer_expiration(TimerWheel *self, TWEntry *entry);
 
-void timer_wheel_set_time(TimerWheel *self, guint64 new_now);
+void timer_wheel_set_time(TimerWheel *self, guint64 new_now, gpointer caller_context);
 guint64 timer_wheel_get_time(TimerWheel *self);
-void timer_wheel_expire_all(TimerWheel *self);
+void timer_wheel_expire_all(TimerWheel *self, gpointer caller_context);
 void timer_wheel_set_associated_data(TimerWheel *self, gpointer assoc_data, GDestroyNotify assoc_data_free);
 gpointer timer_wheel_get_associated_data(TimerWheel *self);
 TimerWheel *timer_wheel_new(void);
