@@ -115,8 +115,6 @@ _init(LogPipe *s)
   if (!log_threaded_source_driver_init_method(s))
     return FALSE;
 
-  log_threaded_source_driver_set_worker_run_func(&self->super, _run);
-  log_threaded_source_driver_set_worker_request_exit_func(&self->super, _request_exit);
 
   return TRUE;
 }
@@ -177,5 +175,8 @@ threaded_random_generator_sd_new(GlobalConfig *cfg)
   self->super.super.super.super.init = _init;
   self->super.format_stats_instance = _format_stats_instance;
 
+  self->super.run = _run;
+  self->super.request_exit = _request_exit;
+
   return &self->super.super.super;
 }
