@@ -61,7 +61,7 @@ extern int hook_commands_debug;
     HOOK_COMMANDS_EMPTY = -2,
     HOOK_COMMANDS_EOF = 0,         /* "end of file"  */
     HOOK_COMMANDS_error = 256,     /* error  */
-    HOOK_COMMANDS_UNDEF = 10514,   /* "invalid token"  */
+    HOOK_COMMANDS_UNDEF = 10523,   /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int hook_commands_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,14 +214,15 @@ extern int hook_commands_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_HOOK_COMMANDS = 10515,      /* KW_HOOK_COMMANDS  */
-    KW_STARTUP = 10516,            /* KW_STARTUP  */
-    KW_SETUP = 10517,              /* KW_SETUP  */
-    KW_TEARDOWN = 10518,           /* KW_TEARDOWN  */
-    KW_SHUTDOWN = 10519            /* KW_SHUTDOWN  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_HOOK_COMMANDS = 10524,      /* KW_HOOK_COMMANDS  */
+    KW_STARTUP = 10525,            /* KW_STARTUP  */
+    KW_SETUP = 10526,              /* KW_SETUP  */
+    KW_TEARDOWN = 10527,           /* KW_TEARDOWN  */
+    KW_SHUTDOWN = 10528            /* KW_SHUTDOWN  */
   };
   typedef enum hook_commands_tokentype hook_commands_token_kind_t;
 #endif
@@ -228,7 +230,7 @@ extern int hook_commands_debug;
 #define HOOK_COMMANDS_EMPTY -2
 #define HOOK_COMMANDS_EOF 0
 #define HOOK_COMMANDS_error 256
-#define HOOK_COMMANDS_UNDEF 10514
+#define HOOK_COMMANDS_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -370,6 +372,7 @@ extern int hook_commands_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -380,14 +383,15 @@ extern int hook_commands_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_HOOK_COMMANDS 10515
-#define KW_STARTUP 10516
-#define KW_SETUP 10517
-#define KW_TEARDOWN 10518
-#define KW_SHUTDOWN 10519
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_HOOK_COMMANDS 10524
+#define KW_STARTUP 10525
+#define KW_SETUP 10526
+#define KW_TEARDOWN 10527
+#define KW_SHUTDOWN 10528
 
 /* Value type.  */
 #if ! defined HOOK_COMMANDS_STYPE && ! defined HOOK_COMMANDS_STYPE_IS_DECLARED
