@@ -69,7 +69,7 @@ extern int kafka_debug;
     KAFKA_EMPTY = -2,
     KAFKA_EOF = 0,                 /* "end of file"  */
     KAFKA_error = 256,             /* error  */
-    KAFKA_UNDEF = 10514,           /* "invalid token"  */
+    KAFKA_UNDEF = 10523,           /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -211,6 +211,7 @@ extern int kafka_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -221,18 +222,19 @@ extern int kafka_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_KAFKA = 10515,              /* KW_KAFKA  */
-    KW_TOPIC = 10516,              /* KW_TOPIC  */
-    KW_FALLBACK_TOPIC = 10517,     /* KW_FALLBACK_TOPIC  */
-    KW_CONFIG = 10518,             /* KW_CONFIG  */
-    KW_FLUSH_TIMEOUT_ON_SHUTDOWN = 10519, /* KW_FLUSH_TIMEOUT_ON_SHUTDOWN  */
-    KW_FLUSH_TIMEOUT_ON_RELOAD = 10520, /* KW_FLUSH_TIMEOUT_ON_RELOAD  */
-    KW_POLL_TIMEOUT = 10521,       /* KW_POLL_TIMEOUT  */
-    KW_BOOTSTRAP_SERVERS = 10522,  /* KW_BOOTSTRAP_SERVERS  */
-    KW_SYNC_SEND = 10523           /* KW_SYNC_SEND  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_KAFKA = 10524,              /* KW_KAFKA  */
+    KW_TOPIC = 10525,              /* KW_TOPIC  */
+    KW_FALLBACK_TOPIC = 10526,     /* KW_FALLBACK_TOPIC  */
+    KW_CONFIG = 10527,             /* KW_CONFIG  */
+    KW_FLUSH_TIMEOUT_ON_SHUTDOWN = 10528, /* KW_FLUSH_TIMEOUT_ON_SHUTDOWN  */
+    KW_FLUSH_TIMEOUT_ON_RELOAD = 10529, /* KW_FLUSH_TIMEOUT_ON_RELOAD  */
+    KW_POLL_TIMEOUT = 10530,       /* KW_POLL_TIMEOUT  */
+    KW_BOOTSTRAP_SERVERS = 10531,  /* KW_BOOTSTRAP_SERVERS  */
+    KW_SYNC_SEND = 10532           /* KW_SYNC_SEND  */
   };
   typedef enum kafka_tokentype kafka_token_kind_t;
 #endif
@@ -240,7 +242,7 @@ extern int kafka_debug;
 #define KAFKA_EMPTY -2
 #define KAFKA_EOF 0
 #define KAFKA_error 256
-#define KAFKA_UNDEF 10514
+#define KAFKA_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -382,6 +384,7 @@ extern int kafka_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -392,18 +395,19 @@ extern int kafka_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_KAFKA 10515
-#define KW_TOPIC 10516
-#define KW_FALLBACK_TOPIC 10517
-#define KW_CONFIG 10518
-#define KW_FLUSH_TIMEOUT_ON_SHUTDOWN 10519
-#define KW_FLUSH_TIMEOUT_ON_RELOAD 10520
-#define KW_POLL_TIMEOUT 10521
-#define KW_BOOTSTRAP_SERVERS 10522
-#define KW_SYNC_SEND 10523
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_KAFKA 10524
+#define KW_TOPIC 10525
+#define KW_FALLBACK_TOPIC 10526
+#define KW_CONFIG 10527
+#define KW_FLUSH_TIMEOUT_ON_SHUTDOWN 10528
+#define KW_FLUSH_TIMEOUT_ON_RELOAD 10529
+#define KW_POLL_TIMEOUT 10530
+#define KW_BOOTSTRAP_SERVERS 10531
+#define KW_SYNC_SEND 10532
 
 /* Value type.  */
 #if ! defined KAFKA_STYPE && ! defined KAFKA_STYPE_IS_DECLARED
