@@ -61,7 +61,7 @@ extern int kv_parser_debug;
     KV_PARSER_EMPTY = -2,
     KV_PARSER_EOF = 0,             /* "end of file"  */
     KV_PARSER_error = 256,         /* error  */
-    KV_PARSER_UNDEF = 10514,       /* "invalid token"  */
+    KV_PARSER_UNDEF = 10523,       /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int kv_parser_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,16 +214,17 @@ extern int kv_parser_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_KV_PARSER = 10515,          /* KW_KV_PARSER  */
-    KW_LINUX_AUDIT_PARSER = 10516, /* KW_LINUX_AUDIT_PARSER  */
-    KW_PREFIX = 10517,             /* KW_PREFIX  */
-    KW_VALUE_SEPARATOR = 10518,    /* KW_VALUE_SEPARATOR  */
-    KW_PAIR_SEPARATOR = 10519,     /* KW_PAIR_SEPARATOR  */
-    KW_EXTRACT_STRAY_WORDS_INTO = 10520, /* KW_EXTRACT_STRAY_WORDS_INTO  */
-    KW_ALLOW_PAIR_SEPARATOR_OPTION = 10521 /* KW_ALLOW_PAIR_SEPARATOR_OPTION  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_KV_PARSER = 10524,          /* KW_KV_PARSER  */
+    KW_LINUX_AUDIT_PARSER = 10525, /* KW_LINUX_AUDIT_PARSER  */
+    KW_PREFIX = 10526,             /* KW_PREFIX  */
+    KW_VALUE_SEPARATOR = 10527,    /* KW_VALUE_SEPARATOR  */
+    KW_PAIR_SEPARATOR = 10528,     /* KW_PAIR_SEPARATOR  */
+    KW_EXTRACT_STRAY_WORDS_INTO = 10529, /* KW_EXTRACT_STRAY_WORDS_INTO  */
+    KW_ALLOW_PAIR_SEPARATOR_OPTION = 10530 /* KW_ALLOW_PAIR_SEPARATOR_OPTION  */
   };
   typedef enum kv_parser_tokentype kv_parser_token_kind_t;
 #endif
@@ -230,7 +232,7 @@ extern int kv_parser_debug;
 #define KV_PARSER_EMPTY -2
 #define KV_PARSER_EOF 0
 #define KV_PARSER_error 256
-#define KV_PARSER_UNDEF 10514
+#define KV_PARSER_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -372,6 +374,7 @@ extern int kv_parser_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -382,16 +385,17 @@ extern int kv_parser_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_KV_PARSER 10515
-#define KW_LINUX_AUDIT_PARSER 10516
-#define KW_PREFIX 10517
-#define KW_VALUE_SEPARATOR 10518
-#define KW_PAIR_SEPARATOR 10519
-#define KW_EXTRACT_STRAY_WORDS_INTO 10520
-#define KW_ALLOW_PAIR_SEPARATOR_OPTION 10521
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_KV_PARSER 10524
+#define KW_LINUX_AUDIT_PARSER 10525
+#define KW_PREFIX 10526
+#define KW_VALUE_SEPARATOR 10527
+#define KW_PAIR_SEPARATOR 10528
+#define KW_EXTRACT_STRAY_WORDS_INTO 10529
+#define KW_ALLOW_PAIR_SEPARATOR_OPTION 10530
 
 /* Value type.  */
 #if ! defined KV_PARSER_STYPE && ! defined KV_PARSER_STYPE_IS_DECLARED
