@@ -69,7 +69,7 @@ extern int mqtt_debug;
     MQTT_EMPTY = -2,
     MQTT_EOF = 0,                  /* "end of file"  */
     MQTT_error = 256,              /* error  */
-    MQTT_UNDEF = 10514,            /* "invalid token"  */
+    MQTT_UNDEF = 10523,            /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -211,6 +211,7 @@ extern int mqtt_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -221,30 +222,33 @@ extern int mqtt_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_MQTT = 10515,               /* KW_MQTT  */
-    KW_TOPIC = 10516,              /* KW_TOPIC  */
-    KW_FALLBACK_TOPIC = 10517,     /* KW_FALLBACK_TOPIC  */
-    KW_KEEPALIVE = 10518,          /* KW_KEEPALIVE  */
-    KW_ADDRESS = 10519,            /* KW_ADDRESS  */
-    KW_QOS = 10520,                /* KW_QOS  */
-    KW_CLIENT_ID = 10521,          /* KW_CLIENT_ID  */
-    KW_CLEANSESSION = 10522,       /* KW_CLEANSESSION  */
-    KW_MESSAGE_TEMPLATE = 10523,   /* KW_MESSAGE_TEMPLATE  */
-    KW_USERNAME = 10524,           /* KW_USERNAME  */
-    KW_PASSWORD = 10525,           /* KW_PASSWORD  */
-    KW_HTTP_PROXY = 10526,         /* KW_HTTP_PROXY  */
-    KW_TLS = 10527,                /* KW_TLS  */
-    KW_CA_DIR = 10528,             /* KW_CA_DIR  */
-    KW_CA_FILE = 10529,            /* KW_CA_FILE  */
-    KW_CERT_FILE = 10530,          /* KW_CERT_FILE  */
-    KW_KEY_FILE = 10531,           /* KW_KEY_FILE  */
-    KW_CIPHER_SUITE = 10532,       /* KW_CIPHER_SUITE  */
-    KW_USE_SYSTEM_CERT_STORE = 10533, /* KW_USE_SYSTEM_CERT_STORE  */
-    KW_SSL_VERSION = 10534,        /* KW_SSL_VERSION  */
-    KW_PEER_VERIFY = 10535         /* KW_PEER_VERIFY  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_MQTT = 10524,               /* KW_MQTT  */
+    KW_TOPIC = 10525,              /* KW_TOPIC  */
+    KW_FALLBACK_TOPIC = 10526,     /* KW_FALLBACK_TOPIC  */
+    KW_KEEPALIVE = 10527,          /* KW_KEEPALIVE  */
+    KW_ADDRESS = 10528,            /* KW_ADDRESS  */
+    KW_QOS = 10529,                /* KW_QOS  */
+    KW_CLIENT_ID = 10530,          /* KW_CLIENT_ID  */
+    KW_CLEANSESSION = 10531,       /* KW_CLEANSESSION  */
+    KW_MESSAGE_TEMPLATE = 10532,   /* KW_MESSAGE_TEMPLATE  */
+    KW_USERNAME = 10533,           /* KW_USERNAME  */
+    KW_PASSWORD = 10534,           /* KW_PASSWORD  */
+    KW_HTTP_PROXY = 10535,         /* KW_HTTP_PROXY  */
+    KW_TLS = 10536,                /* KW_TLS  */
+    KW_CA_DIR = 10537,             /* KW_CA_DIR  */
+    KW_CA_FILE = 10538,            /* KW_CA_FILE  */
+    KW_CERT_FILE = 10539,          /* KW_CERT_FILE  */
+    KW_KEY_FILE = 10540,           /* KW_KEY_FILE  */
+    KW_CIPHER_SUITE = 10541,       /* KW_CIPHER_SUITE  */
+    KW_TLS12_AND_OLDER = 10542,    /* KW_TLS12_AND_OLDER  */
+    KW_TLS13 = 10543,              /* KW_TLS13  */
+    KW_USE_SYSTEM_CERT_STORE = 10544, /* KW_USE_SYSTEM_CERT_STORE  */
+    KW_SSL_VERSION = 10545,        /* KW_SSL_VERSION  */
+    KW_PEER_VERIFY = 10546         /* KW_PEER_VERIFY  */
   };
   typedef enum mqtt_tokentype mqtt_token_kind_t;
 #endif
@@ -252,7 +256,7 @@ extern int mqtt_debug;
 #define MQTT_EMPTY -2
 #define MQTT_EOF 0
 #define MQTT_error 256
-#define MQTT_UNDEF 10514
+#define MQTT_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -394,6 +398,7 @@ extern int mqtt_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -404,30 +409,33 @@ extern int mqtt_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_MQTT 10515
-#define KW_TOPIC 10516
-#define KW_FALLBACK_TOPIC 10517
-#define KW_KEEPALIVE 10518
-#define KW_ADDRESS 10519
-#define KW_QOS 10520
-#define KW_CLIENT_ID 10521
-#define KW_CLEANSESSION 10522
-#define KW_MESSAGE_TEMPLATE 10523
-#define KW_USERNAME 10524
-#define KW_PASSWORD 10525
-#define KW_HTTP_PROXY 10526
-#define KW_TLS 10527
-#define KW_CA_DIR 10528
-#define KW_CA_FILE 10529
-#define KW_CERT_FILE 10530
-#define KW_KEY_FILE 10531
-#define KW_CIPHER_SUITE 10532
-#define KW_USE_SYSTEM_CERT_STORE 10533
-#define KW_SSL_VERSION 10534
-#define KW_PEER_VERIFY 10535
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_MQTT 10524
+#define KW_TOPIC 10525
+#define KW_FALLBACK_TOPIC 10526
+#define KW_KEEPALIVE 10527
+#define KW_ADDRESS 10528
+#define KW_QOS 10529
+#define KW_CLIENT_ID 10530
+#define KW_CLEANSESSION 10531
+#define KW_MESSAGE_TEMPLATE 10532
+#define KW_USERNAME 10533
+#define KW_PASSWORD 10534
+#define KW_HTTP_PROXY 10535
+#define KW_TLS 10536
+#define KW_CA_DIR 10537
+#define KW_CA_FILE 10538
+#define KW_CERT_FILE 10539
+#define KW_KEY_FILE 10540
+#define KW_CIPHER_SUITE 10541
+#define KW_TLS12_AND_OLDER 10542
+#define KW_TLS13 10543
+#define KW_USE_SYSTEM_CERT_STORE 10544
+#define KW_SSL_VERSION 10545
+#define KW_PEER_VERIFY 10546
 
 /* Value type.  */
 #if ! defined MQTT_STYPE && ! defined MQTT_STYPE_IS_DECLARED
