@@ -61,7 +61,7 @@ extern int openbsd_debug;
     OPENBSD_EMPTY = -2,
     OPENBSD_EOF = 0,               /* "end of file"  */
     OPENBSD_error = 256,           /* error  */
-    OPENBSD_UNDEF = 10514,         /* "invalid token"  */
+    OPENBSD_UNDEF = 10523,         /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int openbsd_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,10 +214,11 @@ extern int openbsd_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_OPENBSD = 10515             /* KW_OPENBSD  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_OPENBSD = 10524             /* KW_OPENBSD  */
   };
   typedef enum openbsd_tokentype openbsd_token_kind_t;
 #endif
@@ -224,7 +226,7 @@ extern int openbsd_debug;
 #define OPENBSD_EMPTY -2
 #define OPENBSD_EOF 0
 #define OPENBSD_error 256
-#define OPENBSD_UNDEF 10514
+#define OPENBSD_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -366,6 +368,7 @@ extern int openbsd_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -376,10 +379,11 @@ extern int openbsd_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_OPENBSD 10515
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_OPENBSD 10524
 
 /* Value type.  */
 #if ! defined OPENBSD_STYPE && ! defined OPENBSD_STYPE_IS_DECLARED
