@@ -28,6 +28,8 @@
 #include "str-utils.h"
 #include "string-list.h"
 #include "python-persist.h"
+#include "python-ack-tracker.h"
+#include "python-bookmark.h"
 
 #include <structmember.h>
 
@@ -48,6 +50,7 @@ typedef struct _PythonFetcherDriver
     PyObject *close_method;
     PyObject *request_exit_method;
     PyObject *generate_persist_name;
+    PyAckTrackerFactory *ack_tracker_factory;
   } py;
 } PythonFetcherDriver;
 
@@ -176,6 +179,33 @@ _ulong_to_fetch_result(unsigned long ulong, ThreadedFetchResult *result)
     }
 }
 
+static inline AckTracker *
+_py_fetcher_get_ack_tracker(PythonFetcherDriver *self)
+{
+  return ((LogSource *) self->super.super.worker)->ack_tracker;;
+}
+
+static gboolean
+_py_fetcher_fill_bookmark(PythonFetcherDriver *self, PyLogMessage *pymsg)
+{
+  if (!self->py.ack_tracker_factory)
+    {
+      msg_error("Error in Python fetcher, bookmarks can not be used without creating an AckTracker instance (self.ack_tracker)",
+                evt_tag_str("driver", self->super.super.super.super.id),
+                evt_tag_str("class", self->class));
+      return FALSE;
+    }
+
+  AckTracker *ack_tracker = _py_fetcher_get_ack_tracker(self);
+  Bookmark *bookmark = ack_tracker_request_bookmark(ack_tracker);
+
+  PyBookmark *py_bookmark = py_bookmark_new(pymsg->bookmark_data, self->py.ack_tracker_factory->ack_callback);
+  py_bookmark_fill(bookmark, py_bookmark);
+  Py_XDECREF(py_bookmark);
+
+  return TRUE;
+}
+
 static ThreadedFetchResult
 _py_invoke_fetch(PythonFetcherDriver *self, LogMessage **msg)
 {
@@ -198,6 +228,15 @@ _py_invoke_fetch(PythonFetcherDriver *self, LogMessage **msg)
       if (!pymsg || !py_is_log_message((PyObject *) pymsg))
         goto error;
 
+      if (pymsg->bookmark_data && pymsg->bookmark_data != Py_None)
+        {
+          if (!_py_fetcher_fill_bookmark(self, pymsg))
+            {
+              Py_XDECREF(ret);
+              return THREADED_FETCH_ERROR;
+            }
+        }
+
       /* keep a reference until the PyLogMessage instance is freed */
       *msg = log_msg_ref(pymsg->msg);
     }
@@ -237,6 +276,7 @@ _py_free_bindings(PythonFetcherDriver *self)
   Py_CLEAR(self->py.close_method);
   Py_CLEAR(self->py.request_exit_method);
   Py_CLEAR(self->py.generate_persist_name);
+  Py_CLEAR(self->py.ack_tracker_factory);
 }
 
 static gboolean
@@ -247,11 +287,12 @@ _py_resolve_class(PythonFetcherDriver *self)
   if (!self->py.class)
     {
       gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
 
       msg_error("Error looking Python driver class",
                 evt_tag_str("driver", self->super.super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
       return FALSE;
     }
@@ -267,11 +308,12 @@ _py_init_instance(PythonFetcherDriver *self)
   if (!self->py.instance)
     {
       gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
 
       msg_error("Error instantiating Python driver class",
                 evt_tag_str("driver", self->super.super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
       return FALSE;
     }
@@ -372,11 +414,12 @@ _py_parse_options_new(PythonFetcherDriver *self, MsgFormatOptions *parse_options
   if (!py_parse_options)
     {
       gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
 
       msg_error("Error creating capsule for message parse options",
                 evt_tag_str("driver", self->super.super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
       return NULL;
     }
@@ -384,6 +427,30 @@ _py_parse_options_new(PythonFetcherDriver *self, MsgFormatOptions *parse_options
   return py_parse_options;
 }
 
+static gboolean
+_py_init_ack_tracker_factory(PythonFetcherDriver *self)
+{
+  PyObject *py_ack_tracker_factory = _py_get_attr_or_null(self->py.instance, "ack_tracker");
+
+  if (!py_ack_tracker_factory)
+    return TRUE;
+
+  if (!py_is_ack_tracker_factory(py_ack_tracker_factory))
+    {
+      msg_error("Python source attribute ack_tracker needs to be an AckTracker subtype",
+                evt_tag_str("driver", self->super.super.super.super.id),
+                evt_tag_str("class", self->class));
+      return FALSE;
+    }
+
+  self->py.ack_tracker_factory = (PyAckTrackerFactory *) py_ack_tracker_factory;
+
+  AckTrackerFactory *ack_tracker_factory = self->py.ack_tracker_factory->ack_tracker_factory;
+  self->super.super.worker_options.ack_tracker_factory = ack_tracker_factory_ref(ack_tracker_factory);
+
+  return TRUE;
+}
+
 static gboolean
 _py_set_parse_options(PythonFetcherDriver *self)
 {
@@ -396,11 +463,12 @@ _py_set_parse_options(PythonFetcherDriver *self)
   if (PyObject_SetAttrString(self->py.instance, "parse_options", py_parse_options) == -1)
     {
       gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
 
       msg_error("Error setting attribute message parse options",
                 evt_tag_str("driver", self->super.super.super.super.id),
                 evt_tag_str("class", self->class),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
 
       Py_DECREF(py_parse_options);
@@ -464,6 +532,9 @@ _py_fetcher_init(PythonFetcherDriver *self)
   if (!_py_init_object(self))
     goto error;
 
+  if (!_py_init_ack_tracker_factory(self))
+    goto error;
+
   if (!_py_set_parse_options(self))
     goto error;
 
@@ -533,7 +604,10 @@ python_fetcher_init(LogPipe *s)
               evt_tag_str("driver", self->super.super.super.super.id),
               evt_tag_str("class", self->class));
 
-  return log_threaded_fetcher_driver_init_method(s);
+  if (!log_threaded_fetcher_driver_init_method(s))
+    return FALSE;
+
+  return TRUE;
 }
 
 static gboolean
@@ -541,6 +615,9 @@ python_fetcher_deinit(LogPipe *s)
 {
   PythonFetcherDriver *self = (PythonFetcherDriver *) s;
 
+  AckTracker *ack_tracker = _py_fetcher_get_ack_tracker(self);
+  ack_tracker_deinit(ack_tracker);
+
   PyGILState_STATE gstate = PyGILState_Ensure();
   _py_invoke_deinit(self);
   PyGILState_Release(gstate);
