@@ -77,7 +77,7 @@ extern int python_debug;
     PYTHON_EMPTY = -2,
     PYTHON_EOF = 0,                /* "end of file"  */
     PYTHON_error = 256,            /* error  */
-    PYTHON_UNDEF = 10514,          /* "invalid token"  */
+    PYTHON_UNDEF = 10523,          /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -219,6 +219,7 @@ extern int python_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -229,16 +230,17 @@ extern int python_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_PYTHON = 10515,             /* KW_PYTHON  */
-    KW_PYTHON_FETCHER = 10516,     /* KW_PYTHON_FETCHER  */
-    KW_PYTHON_HTTP_HEADER = 10517, /* KW_PYTHON_HTTP_HEADER  */
-    KW_CLASS = 10518,              /* KW_CLASS  */
-    KW_IMPORTS = 10519,            /* KW_IMPORTS  */
-    KW_LOADERS = 10520,            /* KW_LOADERS  */
-    KW_MARK_ERRORS_AS_CRITICAL = 10521 /* KW_MARK_ERRORS_AS_CRITICAL  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_PYTHON = 10524,             /* KW_PYTHON  */
+    KW_PYTHON_FETCHER = 10525,     /* KW_PYTHON_FETCHER  */
+    KW_PYTHON_HTTP_HEADER = 10526, /* KW_PYTHON_HTTP_HEADER  */
+    KW_CLASS = 10527,              /* KW_CLASS  */
+    KW_IMPORTS = 10528,            /* KW_IMPORTS  */
+    KW_LOADERS = 10529,            /* KW_LOADERS  */
+    KW_MARK_ERRORS_AS_CRITICAL = 10530 /* KW_MARK_ERRORS_AS_CRITICAL  */
   };
   typedef enum python_tokentype python_token_kind_t;
 #endif
@@ -246,7 +248,7 @@ extern int python_debug;
 #define PYTHON_EMPTY -2
 #define PYTHON_EOF 0
 #define PYTHON_error 256
-#define PYTHON_UNDEF 10514
+#define PYTHON_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -388,6 +390,7 @@ extern int python_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -398,16 +401,17 @@ extern int python_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_PYTHON 10515
-#define KW_PYTHON_FETCHER 10516
-#define KW_PYTHON_HTTP_HEADER 10517
-#define KW_CLASS 10518
-#define KW_IMPORTS 10519
-#define KW_LOADERS 10520
-#define KW_MARK_ERRORS_AS_CRITICAL 10521
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_PYTHON 10524
+#define KW_PYTHON_FETCHER 10525
+#define KW_PYTHON_HTTP_HEADER 10526
+#define KW_CLASS 10527
+#define KW_IMPORTS 10528
+#define KW_LOADERS 10529
+#define KW_MARK_ERRORS_AS_CRITICAL 10530
 
 /* Value type.  */
 #if ! defined PYTHON_STYPE && ! defined PYTHON_STYPE_IS_DECLARED
