@@ -48,21 +48,23 @@ static PyObject *
 _py_invoke_template_function(const gchar *function_name, LogMessage *msg, gint argc, GString *argv[])
 {
   PyObject *callable, *ret, *args;
-  gchar buf[256];
 
   callable = _py_resolve_qualified_name(function_name);
   if (!callable)
     {
+      gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
+
       msg_error("$(python): Error looking up Python function",
                 evt_tag_str("function", function_name),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
       return NULL;
     }
 
   msg_debug("$(python): Invoking Python template function",
             evt_tag_str("function", function_name),
-            evt_tag_printf("msg", "%p", msg));
+            evt_tag_msg_reference(msg));
 
   args = _py_construct_args_tuple(msg, argc, argv);
   ret = PyObject_CallObject(callable, args);
@@ -71,9 +73,12 @@ _py_invoke_template_function(const gchar *function_name, LogMessage *msg, gint a
 
   if (!ret)
     {
+      gchar buf[256];
+      _py_format_exception_text(buf, sizeof(buf));
+
       msg_error("$(python): Error invoking Python function",
                 evt_tag_str("function", function_name),
-                evt_tag_str("exception", _py_format_exception_text(buf, sizeof(buf))));
+                evt_tag_str("exception", buf));
       _py_finish_exception_handling();
       return NULL;
     }
@@ -98,12 +103,13 @@ _py_convert_return_value_to_result(const gchar *function_name, PyObject *ret, GS
 }
 
 static void
-tf_python(LogMessage *msg, gint argc, GString *argv[], GString *result)
+tf_python(LogMessage *msg, gint argc, GString *argv[], GString *result, LogMessageValueType *type)
 {
   PyGILState_STATE gstate;
   const gchar *function_name;
   PyObject *ret;
 
+  *type = LM_VT_STRING;
   if (argc == 0)
     return;
   function_name = argv[0]->str;
