@@ -69,7 +69,7 @@ extern int redis_debug;
     REDIS_EMPTY = -2,
     REDIS_EOF = 0,                 /* "end of file"  */
     REDIS_error = 256,             /* error  */
-    REDIS_UNDEF = 10514,           /* "invalid token"  */
+    REDIS_UNDEF = 10523,           /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -211,6 +211,7 @@ extern int redis_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -221,13 +222,14 @@ extern int redis_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_REDIS = 10515,              /* KW_REDIS  */
-    KW_COMMAND = 10516,            /* KW_COMMAND  */
-    KW_AUTH = 10517,               /* KW_AUTH  */
-    KW_TIMEOUT = 10518             /* KW_TIMEOUT  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_REDIS = 10524,              /* KW_REDIS  */
+    KW_COMMAND = 10525,            /* KW_COMMAND  */
+    KW_AUTH = 10526,               /* KW_AUTH  */
+    KW_TIMEOUT = 10527             /* KW_TIMEOUT  */
   };
   typedef enum redis_tokentype redis_token_kind_t;
 #endif
@@ -235,7 +237,7 @@ extern int redis_debug;
 #define REDIS_EMPTY -2
 #define REDIS_EOF 0
 #define REDIS_error 256
-#define REDIS_UNDEF 10514
+#define REDIS_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -377,6 +379,7 @@ extern int redis_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -387,13 +390,14 @@ extern int redis_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_REDIS 10515
-#define KW_COMMAND 10516
-#define KW_AUTH 10517
-#define KW_TIMEOUT 10518
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_REDIS 10524
+#define KW_COMMAND 10525
+#define KW_AUTH 10526
+#define KW_TIMEOUT 10527
 
 /* Value type.  */
 #if ! defined REDIS_STYPE && ! defined REDIS_STYPE_IS_DECLARED
