@@ -70,7 +70,7 @@ extern int riemann_debug;
     RIEMANN_EMPTY = -2,
     RIEMANN_EOF = 0,               /* "end of file"  */
     RIEMANN_error = 256,           /* error  */
-    RIEMANN_UNDEF = 10514,         /* "invalid token"  */
+    RIEMANN_UNDEF = 10523,         /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -212,6 +212,7 @@ extern int riemann_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -222,25 +223,26 @@ extern int riemann_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_RIEMANN = 10515,            /* KW_RIEMANN  */
-    KW_SERVER = 10516,             /* KW_SERVER  */
-    KW_STATE = 10517,              /* KW_STATE  */
-    KW_SERVICE = 10518,            /* KW_SERVICE  */
-    KW_EVENT_TIME = 10519,         /* KW_EVENT_TIME  */
-    KW_SECONDS = 10520,            /* KW_SECONDS  */
-    KW_MICROSECONDS = 10521,       /* KW_MICROSECONDS  */
-    KW_DESCRIPTION = 10522,        /* KW_DESCRIPTION  */
-    KW_METRIC = 10523,             /* KW_METRIC  */
-    KW_TTL = 10524,                /* KW_TTL  */
-    KW_ATTRIBUTES = 10525,         /* KW_ATTRIBUTES  */
-    KW_TIMEOUT = 10526,            /* KW_TIMEOUT  */
-    KW_CA_FILE = 10527,            /* KW_CA_FILE  */
-    KW_CERT_FILE = 10528,          /* KW_CERT_FILE  */
-    KW_KEY_FILE = 10529,           /* KW_KEY_FILE  */
-    KW_TLS = 10530                 /* KW_TLS  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_RIEMANN = 10524,            /* KW_RIEMANN  */
+    KW_SERVER = 10525,             /* KW_SERVER  */
+    KW_STATE = 10526,              /* KW_STATE  */
+    KW_SERVICE = 10527,            /* KW_SERVICE  */
+    KW_EVENT_TIME = 10528,         /* KW_EVENT_TIME  */
+    KW_SECONDS = 10529,            /* KW_SECONDS  */
+    KW_MICROSECONDS = 10530,       /* KW_MICROSECONDS  */
+    KW_DESCRIPTION = 10531,        /* KW_DESCRIPTION  */
+    KW_METRIC = 10532,             /* KW_METRIC  */
+    KW_TTL = 10533,                /* KW_TTL  */
+    KW_ATTRIBUTES = 10534,         /* KW_ATTRIBUTES  */
+    KW_TIMEOUT = 10535,            /* KW_TIMEOUT  */
+    KW_CA_FILE = 10536,            /* KW_CA_FILE  */
+    KW_CERT_FILE = 10537,          /* KW_CERT_FILE  */
+    KW_KEY_FILE = 10538,           /* KW_KEY_FILE  */
+    KW_TLS = 10539                 /* KW_TLS  */
   };
   typedef enum riemann_tokentype riemann_token_kind_t;
 #endif
@@ -248,7 +250,7 @@ extern int riemann_debug;
 #define RIEMANN_EMPTY -2
 #define RIEMANN_EOF 0
 #define RIEMANN_error 256
-#define RIEMANN_UNDEF 10514
+#define RIEMANN_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -390,6 +392,7 @@ extern int riemann_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -400,25 +403,26 @@ extern int riemann_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_RIEMANN 10515
-#define KW_SERVER 10516
-#define KW_STATE 10517
-#define KW_SERVICE 10518
-#define KW_EVENT_TIME 10519
-#define KW_SECONDS 10520
-#define KW_MICROSECONDS 10521
-#define KW_DESCRIPTION 10522
-#define KW_METRIC 10523
-#define KW_TTL 10524
-#define KW_ATTRIBUTES 10525
-#define KW_TIMEOUT 10526
-#define KW_CA_FILE 10527
-#define KW_CERT_FILE 10528
-#define KW_KEY_FILE 10529
-#define KW_TLS 10530
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_RIEMANN 10524
+#define KW_SERVER 10525
+#define KW_STATE 10526
+#define KW_SERVICE 10527
+#define KW_EVENT_TIME 10528
+#define KW_SECONDS 10529
+#define KW_MICROSECONDS 10530
+#define KW_DESCRIPTION 10531
+#define KW_METRIC 10532
+#define KW_TTL 10533
+#define KW_ATTRIBUTES 10534
+#define KW_TIMEOUT 10535
+#define KW_CA_FILE 10536
+#define KW_CERT_FILE 10537
+#define KW_KEY_FILE 10538
+#define KW_TLS 10539
 
 /* Value type.  */
 #if ! defined RIEMANN_STYPE && ! defined RIEMANN_STYPE_IS_DECLARED
