@@ -25,6 +25,8 @@
 #include "riemann.h"
 #include "riemann-worker.h"
 #include "scratch-buffers.h"
+#include "generic-number.h"
+#include "parse-number.h"
 
 #include <riemann/simple.h>
 #include <stdlib.h>
@@ -99,7 +101,7 @@ riemann_dd_field_string_maybe_add(riemann_event_t *event, LogMessage *msg,
     return;
 
   LogTemplateEvalOptions options = {template_options, LTZ_SEND,
-                                    seq_num, NULL
+                                    seq_num, NULL, LM_VT_STRING
                                    };
   log_template_format(template, msg, &options, target);
 
@@ -118,14 +120,16 @@ riemann_dd_field_integer_maybe_add(riemann_event_t *event, LogMessage *msg,
     return;
 
   LogTemplateEvalOptions options = {template_options, LTZ_SEND,
-                                    seq_num, NULL
+                                    seq_num, NULL, LM_VT_STRING
                                    };
   log_template_format(template, msg, &options, target);
 
   if (target->len != 0)
     {
-      gint64 as_int = g_ascii_strtoll(target->str, NULL, 10);
-      riemann_event_set(event, ftype, as_int, RIEMANN_EVENT_FIELD_NONE);
+      GenericNumber gn;
+
+      if (parse_generic_number(target->str, &gn))
+        riemann_event_set(event, ftype, gn_as_int64(&gn), RIEMANN_EVENT_FIELD_NONE);
     }
 }
 
@@ -153,7 +157,7 @@ riemann_dd_field_add_msg_tag(const LogMessage *msg,
 /* TODO escape '\0' when passing down the value */
 static gboolean
 riemann_dd_field_add_attribute_vp(const gchar *name,
-                                  TypeHint type, const gchar *value,
+                                  LogMessageValueType type, const gchar *value,
                                   gsize value_len,
                                   gpointer user_data)
 {
@@ -170,19 +174,20 @@ static gboolean
 riemann_add_metric_to_event(RiemannDestWorker *self, riemann_event_t *event, LogMessage *msg, GString *str)
 {
   RiemannDestDriver *owner = (RiemannDestDriver *) self->super.owner;
+  LogMessageValueType type;
 
   LogTemplateEvalOptions options = {&owner->template_options,
-                                    LTZ_SEND, self->super.seq_num, NULL
+                                    LTZ_SEND, self->super.seq_num, NULL, LM_VT_STRING
                                    };
-  log_template_format(owner->fields.metric, msg, &options, str);
+  log_template_format_value_and_type(owner->fields.metric, msg, &options, str, &type);
 
   if (str->len == 0)
     return TRUE;
 
-  switch (owner->fields.metric->type_hint)
+  switch (type)
     {
-    case TYPE_HINT_INT32:
-    case TYPE_HINT_INT64:
+    case LM_VT_INT32:
+    case LM_VT_INT64:
     {
       gint64 i;
 
@@ -194,8 +199,8 @@ riemann_add_metric_to_event(RiemannDestWorker *self, riemann_event_t *event, Log
                                       str->str, "int");
       break;
     }
-    case TYPE_HINT_DOUBLE:
-    case TYPE_HINT_STRING:
+    case LM_VT_DOUBLE:
+    case LM_VT_STRING:
     {
       gdouble d;
 
@@ -222,7 +227,7 @@ riemann_add_ttl_to_event(RiemannDestWorker *self, riemann_event_t *event, LogMes
   gdouble d;
 
   LogTemplateEvalOptions options = {&owner->template_options,
-                                    LTZ_SEND, self->super.seq_num, NULL
+                                    LTZ_SEND, self->super.seq_num, NULL, LM_VT_STRING
                                    };
   log_template_format(owner->fields.ttl, msg, &options, str);
 
@@ -299,7 +304,7 @@ riemann_worker_insert_one(RiemannDestWorker *self, LogMessage *msg)
 
       if (owner->fields.attributes)
         {
-          LogTemplateEvalOptions options = {&owner->template_options, LTZ_SEND, self->super.seq_num, NULL};
+          LogTemplateEvalOptions options = {&owner->template_options, LTZ_SEND, self->super.seq_num, NULL, LM_VT_STRING};
           value_pairs_foreach(owner->fields.attributes,
                               riemann_dd_field_add_attribute_vp,
                               msg, &options, event);
