@@ -61,7 +61,7 @@ extern int xml_debug;
     XML_EMPTY = -2,
     XML_EOF = 0,                   /* "end of file"  */
     XML_error = 256,               /* error  */
-    XML_UNDEF = 10514,             /* "invalid token"  */
+    XML_UNDEF = 10523,             /* "invalid token"  */
     LL_CONTEXT_ROOT = 1,           /* LL_CONTEXT_ROOT  */
     LL_CONTEXT_DESTINATION = 2,    /* LL_CONTEXT_DESTINATION  */
     LL_CONTEXT_SOURCE = 3,         /* LL_CONTEXT_SOURCE  */
@@ -203,6 +203,7 @@ extern int xml_debug;
     LL_STRING = 10433,             /* LL_STRING  */
     LL_TOKEN = 10434,              /* LL_TOKEN  */
     LL_BLOCK = 10435,              /* LL_BLOCK  */
+    LL_PLUGIN = 10436,             /* LL_PLUGIN  */
     KW_VALUE_PAIRS = 10500,        /* KW_VALUE_PAIRS  */
     KW_EXCLUDE = 10502,            /* KW_EXCLUDE  */
     KW_PAIR = 10503,               /* KW_PAIR  */
@@ -213,15 +214,16 @@ extern int xml_debug;
     KW_REKEY = 10508,              /* KW_REKEY  */
     KW_ADD_PREFIX = 10509,         /* KW_ADD_PREFIX  */
     KW_REPLACE_PREFIX = 10510,     /* KW_REPLACE_PREFIX  */
-    KW_ON_ERROR = 10511,           /* KW_ON_ERROR  */
-    KW_RETRIES = 10512,            /* KW_RETRIES  */
-    KW_FETCH_NO_DATA_DELAY = 10513, /* KW_FETCH_NO_DATA_DELAY  */
-    KW_XML = 10515,                /* KW_XML  */
-    KW_PREFIX = 10516,             /* KW_PREFIX  */
-    KW_DROP_INVALID = 10517,       /* KW_DROP_INVALID  */
-    KW_EXCLUDE_TAGS = 10518,       /* KW_EXCLUDE_TAGS  */
-    KW_STRIP_WHITESPACES = 10519,  /* KW_STRIP_WHITESPACES  */
-    KW_CREATE_LISTS = 10520        /* KW_CREATE_LISTS  */
+    KW_CAST = 10511,               /* KW_CAST  */
+    KW_ON_ERROR = 10520,           /* KW_ON_ERROR  */
+    KW_RETRIES = 10521,            /* KW_RETRIES  */
+    KW_FETCH_NO_DATA_DELAY = 10522, /* KW_FETCH_NO_DATA_DELAY  */
+    KW_XML = 10524,                /* KW_XML  */
+    KW_PREFIX = 10525,             /* KW_PREFIX  */
+    KW_DROP_INVALID = 10526,       /* KW_DROP_INVALID  */
+    KW_EXCLUDE_TAGS = 10527,       /* KW_EXCLUDE_TAGS  */
+    KW_STRIP_WHITESPACES = 10528,  /* KW_STRIP_WHITESPACES  */
+    KW_CREATE_LISTS = 10529        /* KW_CREATE_LISTS  */
   };
   typedef enum xml_tokentype xml_token_kind_t;
 #endif
@@ -229,7 +231,7 @@ extern int xml_debug;
 #define XML_EMPTY -2
 #define XML_EOF 0
 #define XML_error 256
-#define XML_UNDEF 10514
+#define XML_UNDEF 10523
 #define LL_CONTEXT_ROOT 1
 #define LL_CONTEXT_DESTINATION 2
 #define LL_CONTEXT_SOURCE 3
@@ -371,6 +373,7 @@ extern int xml_debug;
 #define LL_STRING 10433
 #define LL_TOKEN 10434
 #define LL_BLOCK 10435
+#define LL_PLUGIN 10436
 #define KW_VALUE_PAIRS 10500
 #define KW_EXCLUDE 10502
 #define KW_PAIR 10503
@@ -381,15 +384,16 @@ extern int xml_debug;
 #define KW_REKEY 10508
 #define KW_ADD_PREFIX 10509
 #define KW_REPLACE_PREFIX 10510
-#define KW_ON_ERROR 10511
-#define KW_RETRIES 10512
-#define KW_FETCH_NO_DATA_DELAY 10513
-#define KW_XML 10515
-#define KW_PREFIX 10516
-#define KW_DROP_INVALID 10517
-#define KW_EXCLUDE_TAGS 10518
-#define KW_STRIP_WHITESPACES 10519
-#define KW_CREATE_LISTS 10520
+#define KW_CAST 10511
+#define KW_ON_ERROR 10520
+#define KW_RETRIES 10521
+#define KW_FETCH_NO_DATA_DELAY 10522
+#define KW_XML 10524
+#define KW_PREFIX 10525
+#define KW_DROP_INVALID 10526
+#define KW_EXCLUDE_TAGS 10527
+#define KW_STRIP_WHITESPACES 10528
+#define KW_CREATE_LISTS 10529
 
 /* Value type.  */
 #if ! defined XML_STYPE && ! defined XML_STYPE_IS_DECLARED
