@@ -32,19 +32,13 @@ news_dir = Path(__file__).resolve().parent
 root_dir = news_dir.parent
 newsfile = root_dir / 'NEWS.md'
 
-team_members = [
+credit_fixed_contributors = [
     "Andras Mitzki",
-    "Attila Szakacs",
-    "Balázs Barkó",
     "Balazs Scheidler",
-    "Benedek Cserhati",
     "Gabor Nagy",
-    "Laszlo Szemere",
     "László Várady",
-    "Norbert Takacs",
     "Parrag Szilárd",
     "Peter Kokai",
-    "Zoltan Pallagi",
 ]
 exclude_contributor_list = [
     "github-actions",
@@ -82,8 +76,9 @@ def create_block(block_name, files):
 
         pr_id = match_pr_id.group(1)
 
-        entry += ' * {}\n([#{}](https://github.com/syslog-ng/syslog-ng/pull/{}))'.format(f.read_text().rstrip(), pr_id, pr_id)
-        entry = entry.replace('\n', '\n   ')
+        entry += '  * {}\n([#{}](https://github.com/syslog-ng/syslog-ng/pull/{}))'.format(f.read_text().rstrip(), pr_id, pr_id)
+        entry = entry.replace('\n', '\n    ')
+        entry = entry.replace('\n    \n', '\n\n')
         block += entry + '\n'
     block += '\n'
     return block
@@ -128,7 +123,7 @@ def create_credits_block():
     stdout = _exec(r'git rev-list --no-merges --format=format:%aN syslog-ng-' + get_last_version() + r'..HEAD | '
                    r'grep -Ev "^commit [a-z0-9]{40}$" | sort | uniq')
     contributors = stdout.rstrip().split('\n')
-    contributors += team_members
+    contributors += credit_fixed_contributors
     contributors = filter(lambda x : x not in exclude_contributor_list, contributors)
     contributors = sorted(set(contributors))
 
