@@ -26,13 +26,14 @@
 #define CONTROL_CLIENT_H 1
 
 #include "syslog-ng.h"
+#include "commands/commands.h"
 
 typedef struct _ControlClient ControlClient;
 
 ControlClient *control_client_new(const gchar *path);
 gboolean control_client_connect(ControlClient *self);
 gint control_client_send_command(ControlClient *self, const gchar *cmd);
-GString *control_client_read_reply(ControlClient *self);
+gint control_client_read_reply(ControlClient *self, CommandResponseHandlerFunc cb, gpointer user_data);
 void control_client_free(ControlClient *self);
 
 #endif
